DEFINE CLASS CONVERTMODULE_CONTROL AS Custom
* -------------------------------------------------------------------------------- *
*	CONVERTMODULE_CONTROL
*	=====================
*	Desc (short):	ConvertModule_Control implements the default creation of the 
*					source classes that allow the data transformation into other 
*					formats
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.03 - 2005-06-21
*	VFP Version.:	VFP 7.0
*	Description.:	This class allows to call the specialized MS-Excel, 
*					OpenOffice,... convert methods which read the data and put
*					the results into a temporary table (also called cursor file).
*					From there a postprocessing routine is called to make some
*					changes (=> user code - see Wizard and program documentation).
*					After that the data from the temporary table is exported into
*					the selected format.
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-06-15 - J.U. -	work started
*						2005-06-20 - J.U. -	updated call of SetupId of oAppControl in main() to 
*											submit the basic information about the types and
*											the selected import/export definitions
*						2005-06-21 - J.U. -	added debug parameter to setup, forward debug setting to 
*											oAppControl for debugging purposes, cleaned up code and 
*											updated method-list
*						2005-09-16 - J.U. - added support to file extensions in open and save dialog
*													
*	Methods.....:	init() returns logical
*					setup([toMsgProgressBar,][toSegProgressBar,][tlDebugMode]) returns logical
*					main() returns logical (only executed if this.lSetupOk is true)
*                 	                                                               
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF


****************
* SET BY SETUP *
****************
PROTECTED lSetupOk					&& Was the Setup ok?
PROTECTED oMsgProgressBar			&& ProgressBar
PROTECTED oSegProgressBar			&& ProgressBar
PROTECTED lDebugMode				&& DebugMode


******************
* SET BY SETUPID *
******************
PROTECTED nTypeId					&& Id for the Types table entry
PROTECTED nImpFileId				&& Id for the Importfile table entry
PROTECTED nExpFileId				&& Id for the Exportfile table entry


****************
*    OTHER     *
****************
PROTECTED oAppControl				&& Application Controller object


****************
*    TABLES    *
****************
cTypesSel            = ""
cImportFileSel       = ""
cImportFieldDefSel   = ""
cExportFileSel       = ""
cExportFieldDefSel   = ""
cMappingSel          = ""


*!* Init for this class 
*!*
PROCEDURE init
   DODEFAULT()
   this.cCRLF           = CHR(13)+CHR(10) 
   this.lSetupOk        = .F.   
RETURN
ENDPROC


*!* Setup for this class 
*!*
*!* Required Parameter(s):
*!* NONE
*!*	
*!* Optional Parameter(s):     
*!*	toMsgProgressBar	= ProgressBar (default is none)
*!*	toSegProgressBar	= ProgressBar (default is none)
*!* tlDebugMode			= Enable or Disable DebugMode (default is off)
*!*
PROCEDURE setup
LPARAMETERS toMsgProgressBar, toSegProgressBar, tlDebugMode
LOCAL llok
	llok = .T.
	
	IF llok AND VARTYPE(toMsgProgressBar) == "O"
		this.oMsgProgressBar = toMsgProgressBar&& set MsgProgressBar reference from Parameter
	ENDIF

	IF llok AND VARTYPE(toSegProgressBar) == "O"
		this.oSegProgressBar = toSegProgressBar&& set SegProgressBar reference from Parameter
	ENDIF
	
	IF llok AND VARTYPE(tlDebugMode) == "L"
		this.lDebugMode = tlDebugMode
	ELSE
		this.lDebugMode = .F.
	ENDIF
   
   this.lSetupOk = llok && set a marker to show if the setup returned ok or not
RETURN(llok)
ENDPROC





*!* *************************
*!* MAIN METHOD of this class 
*!* *************************
PROCEDURE main
LOCAL llOk, lcOldSel, lcTmpHelpSel, lcImportType, lnImportTypeId, lnImportFileId, lcImportFileName
LOCAL lcImportType, lnImportTypeId, lnExportFileId, lcExportFileName, lImportTypeFound, lExportTypeFound
LOCAL lcExportExtension, lcImportExtension
	llOk = .T.
	lcOldSel  = SELECT(0) && save the currently selected cursor
	lcTmpHelpSel     = "" && helps to receive information from different tables...the cursor incl. the needed field is used here

	lcImportType     = ""
	lnImportTypeId   = 0
	
	lnImportFileId   = 0
	lcImportFileName = ""

	lcImportExtension = "" && File extension (DBF|XLS|XML|TXT|SXC)

	lcExportType     = ""
	lnExportTypeId   = 0
	
	lnExportFileId   = 0
	lcExportFileName = ""

	lcExportExtension = "" && File extension (DBF|XLS|XML|TXT|SXC)
	
	lImportTypeFound = .F. && could the import type be found in the type cursor?
	lExportTypeFound = .F. && could the export type be found in the type cursor?
	

	***************
	* IMPORT FILE *
	***************
	&& get id field from ImportFile
	lcTmpHelpSel = this.cImportFileSel + ".IMPFILEID"
	lnImportFileId = &lcTmpHelpSel


	&& get TYPE id field from ImportFile
	lcTmpHelpSel = this.cImportFileSel + ".TYPEID"
	lnImportTypeId = &lcTmpHelpSel
	
	&& get filename field from ImportFile
	lcTmpHelpSel = this.cImportFileSel + ".FILENAME"
	lcImportFileName = ALLTRIM(&lcTmpHelpSel)

	
	&& look for the import type definition
	SELECT(this.cTypesSel)
	LOCATE FOR typeid = lnImportTypeId
	IF FOUND()
		lImportTypeFound = .T.
		
		lcTmpHelpSel = this.cTypesSel + ".TYPE"
		lcImportType = ALLTRIM(&lcTmpHelpSel)
	
		&& SET FILE EXTENSION
		lcTmpHelpSel = this.cTypesSel + ".EXTENSION"
		lcImportExtension = ALLTRIM(&lcTmpHelpSel)
	
	ELSE  && could not find the entry
		llok = .F.
	ENDIF 		

	&& if the filename is empty() or the file does not exist...ask for the file to open
	IF llOk AND EMPTY(lcImportFileName) OR NOT FILE(lcImportFileName) THEN && displays the Open dialog box
		lcImportFileName = GETFILE("*."+STRTRAN(lcImportExtension, "|", ",*.")+":"+STRTRAN(lcImportExtension, "|", ","))
	ENDIF

	
	IF lImportTypeFound AND NOT EMPTY(lcImportFileName) THEN
		***************
		* EXPORT FILE *
		***************
		&& get id field from ExportFile
		lcTmpHelpSel = this.cExportFileSel + ".EXPFILEID"
		lnExportFileId = &lcTmpHelpSel

		&& get TYPE id field from ExportFile
		lcTmpHelpSel = this.cExportFileSel + ".TYPEID"
		lnExportTypeId = &lcTmpHelpSel
		
		&& get filename field from the ExportFile
		lcTmpHelpSel = this.cExportFileSel + ".FILENAME"
		lcExportFileName = ALLTRIM(&lcTmpHelpSel)
		
		&& look for the export type definition
		SELECT(this.cTypesSel)
		LOCATE FOR typeid = lnExportTypeId
		IF FOUND()
			lExportTypeFound = .T.
			
			lcTmpHelpSel = this.cTypesSel + ".TYPE"
			lcExportType = ALLTRIM(&lcTmpHelpSel)
			
			&& SET FILE EXTENSION
			lcTmpHelpSel = this.cTypesSel + ".EXTENSION"
			lcExportExtension = ALLTRIM(&lcTmpHelpSel)
		ELSE && could not find the entry
			llOk = .F.
		ENDIF 				
		
		IF llok THEN
			&& if the filename is empty() ask for target file (where to save the data)
			IF EMPTY(lcExportFileName) THEN && invoke the Save As dialog box and returns the file name you specify
				lcExportFileName = PUTFILE("Speichern unter...", "VFPDC_Export", lcExportExtension)
*!*				ELSE
*!*					IF FILE(lcExportFileName) THEN
*!*						IF MESSAGEBOX("Die Datei "+lcExportFileName+" existiert bereits!";
*!*							+this.cCRLF+"Soll die Datei berschrieben werden?",292,;
*!*							"Sicherheitsabfrage") == 7 THEN && (7 == NO [DEFAULT selection])
*!*							&& NO = do not overwrite...invoke the Save As dialog box
*!*							lcExportFileName = PUTFILE()
*!*						ENDIF
*!*					ENDIF
			ENDIF
		ENDIF && if..endif llok
	ENDIF && lcImportFileName is not empty
	
	IF llok AND NOT EMPTY(lcImportFileName) AND NOT EMPTY(lcExportFileName) THEN
		&& check if this.oAppControl is already an instance...if not create a new instance of it
		IF NOT EMPTY(lcImportType) AND !VARTYPE(this.loAppControl)=="O"
			IF this.lDebugMode THEN
				WAIT WINDOW ("ClassFile: "+lcImportType+"_CONTROL")
			ENDIF
			
			this.oAppControl = NEWOBJECT(lcImportType+"_CONTROL", lcImportType+"_Control.prg")
			IF VARTYPE(this.oAppControl) == "O" THEN
			
				IF this.lDebugMode THEN
					=MESSAGEBOX("Kontrollinstanz der Anwendung erzeugt...", 64, "Information (ConvertModule_Control.prg)") && INFORMATION
				ENDIF
				
				IF this.oAppControl.setup(lcImportFileName, JUSTFNAME(lcExportFileName), ADDBS(JUSTPATH(lcExportFileName)), "", "", this.lDebugMode) THEN && tcImportFileNameFullPath, tcExportFileName, tcExportPath, toMsgProgressBar, toSegProgressBar, tlDebugMode
					this.oAppControl.setupId(lnImportTypeId, lcImportType, lnImportFileId, lnExportTypeId, lcExportType, lnExportFileId)
				
					this.oAppControl.cTypesSel    = this.cTypesSel
					this.oAppControl.cImpFileSel  = this.cImportFileSel
					this.oAppControl.cImpFieldSel = this.cImportFieldDefSel
					this.oAppControl.cExpFileSel  = this.cExportFileSel
					this.oAppControl.cExpFieldSel = this.cExportFieldDefSel
					this.oAppControl.cMappingSel  = this.cMappingSel
					
					&& =========================================================================
					this.oAppControl.main() && execute import of selected application controller
					&& =========================================================================
					
					this.oAppControl="" && remove object after use
				ELSE
					=MESSAGEBOX("Setup der Kontrollinstanz ist fehlgeschlagen!", 16, "Fehler") && ERROR/STOP SIGN, OK button
				ENDIF
			ELSE
				=MESSAGEBOX("Fehler bei der Erstellung der Instanz der Klasse "+lcType+"_Control!", 16, "Fehler") && ERROR/STOP SIGN, OK button
			ENDIF
		ENDIF && check TYPE and call application control
		
	ENDIF && lcImportFileName and lcExportFileName not empty

	SELECT(lcOldSel) && restore the saved cursor
RETURN(llok)
ENDPROC



ENDDEFINE && END OF CLASS DEFINITION
