DEFINE CLASS EXCEL_CONTROL AS GENERIC_CONTROL OF GENERIC_CONTROL.PRG
* -------------------------------------------------------------------------------- *
*	EXCELCONTROL
*	============
*	Desc (short):	CSV to MS-Excel import and MS-Excel to CSV export 
*					(configurable)
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.11 - 2005-06-07
*	VFP Version.:	VFP 7.0
*	Description.:	This class allows to import data into a cursor and from there 
*					to different output formats
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-04-21 - J.U. -	work started
*						2005-05-26 - J.U. -	most work done; mapping excel fields by table definition to other fields
*											is done
*						2005-06-07 - J.U. -	moved most of the code (the program logic) to GenericControl 
*											and added here only the basic commands that are used from
*											Excel to accomplish the tasks (overwriting the code from GenericControl).
*						2005-06-21 - J.U. -	updated method list
*													
*	Methods.....:	openApplication()
*					closeApplication()
*					openAppWrkDoc(tcImportFile)
*					selectAppWrkDocColumnRow(tnColumn, tnRow)
*					getAppWrkDocSelectedElement()
*					setAppVisible(tlVisible)
*                 	                                                               
* -------------------------------------------------------------------------------- *


*!* Open Application
*!* This function opens the application to be used in the following procedures and functions
*!* MS-Excel is opened here
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
FUNCTION openApplication
LOCAL llok
	llok = DODEFAULT()
	this.oApplication = CREATEOBJECT("excel.application")
	IF VARTYPE(this.oApplication) == "O" THEN
		llOk = .T.
	ELSE
		llOk = .F.
	ENDIF
RETURN(llok)
ENDFUNC


*!* Close Application
*!* This function closes and removes the application that has been used
*!* Close MS-Excel
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
FUNCTION closeApplication
LOCAL llok
	llok = DODEFAULT()
	IF VARTYPE(this.oApplication) == "O" THEN
		this.oApplication.Quit
		this.oApplication = ""
	ENDIF
RETURN(llok)
ENDFUNC



*!* Open Application Work Document
*!* This function opens the working document
*!*
*!* Required Parameter(s):
*!*	tcImportFile	= The input file that has to be opened and used to import the data from it
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION openAppWrkDoc
PARAMETERS tcImportFile
LOCAL llok
	llok = DODEFAULT(tcImportFile)
	IF llok THEN
		this.oApplication.Workbooks.Open(tcImportFile)
	ENDIF
RETURN(llok)
ENDFUNC


*!* Select Application Work Document column and row
*!* This function selects the column and row of the work document
*!*
*!* Required Parameter(s):
*!*	tnColumn	= The column that should be selected
*!* tnRow		= The row that should be selected
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION selectAppWrkDocColumnRow
PARAMETERS tnColumn, tnRow
LOCAL llok
	llok = DODEFAULT(tnColumn, tnRow) && checks parameters are from the correct type
	IF llok THEN
		this.oApplication.Range(this.Number2Spread(tnColumn)+TRANSFORM(tnRow)).Select && select row and column
	ENDIF
RETURN(llok)
ENDFUNC


*!* Get Selected Element of Application Work Document
*!* This function returns the selected element of the work document
*!*
*!* Required Parameter(s):
*!* NONE
*!*
FUNCTION getAppWrkDocSelectedElement
LOCAL lcString
	lcString = DODEFAULT()
	lcString = this.oApplication.ActiveCell.Text
RETURN(lcString)
ENDFUNC


*!* Set Application Visible
*!* This function makes the application visible or invisible to the enduser
*!*
*!* Required Parameter(s):
*!* tlVisible	= Visible TRUE (.T.) or FALSE (.F.)
*!*
FUNCTION setAppVisible
PARAMETERS tlVisible
LOCAL llok
	llok = DODEFAULT(tlVisible) && checks parameter if it's from the correct type
	IF llok THEN
		this.oApplication.visible = tlVisible
	ENDIF
RETURN(llok)
ENDFUNC

ENDDEFINE && END OF CLASS DEFINITION
