DEFINE CLASS GENERIC_CONTROL AS Custom
* -------------------------------------------------------------------------------- *
*	GENERIC_CONTROL
*	===============
*	Desc (short):	Generic_Control implements the default calls every child also 
*					implements + the generic functions and procedures to import 
*					and export data from/into different formats!
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.06 - 2005-09-16
*	VFP Version.:	VFP 9.0
*	Description.:	This class allows to call the basic import data methods
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-06-07 - J.U. -	work started (derived from ExcelControl)
*						2005-06-20 - J.U. -	added export type id (nExpTypeId) and import/export type names 
*											(cImpTypeName/cExpTypeName)
*						2005-09-07 - J.U. - cleaned method and functions descriptions
*											added method list with parameters and returned types
*						2005-09-11 - J.U. - added postprocessing code
*						2005-09-14 - J.U. - renamed postprocessing to preprocessing and added at to the export part
*						2005-09-16 - J.U. - added several export formats (Excel 2/5+, Lotus 1-2-3 v2+, 
*											Visual Foxpro DBF, FoxPro 2.x DBF) and updated 
*											Added functionality to use parameters from export 
*											rep_def memo field to callHttpForm to be able to set 
*											server and path on server
*						2005-10-18 - J.U. - replacement of not valid ANSI characters with the 
*											correct XML named entity at the exportXml_WebDAV_OX function
*						2005-10-19 - J.U. - replacement of 'Vertical Tab' with CR+LF in 
*											exportXml_WebDAV_OX function
*													
*	Methods.....:	init() ---
*					setup(tcImportFile, tcOutFile, tcOutDir[, toMsgProgressBar, toSegProgressBar, tlDebugMode]) boolean
*					setupId(tnImpTypeId, tcImpTypeName, tnImpFileId, tnExpTypeId, tcExpTypeName, tnExpFileId) boolean
*					open_application() boolean
*					close_application() boolean
*					openAppWrkDoc(tcImportFile) boolean
*					selectAppWrkDocColumnRow(tnColumn, tnRow) boolean
*					getAppWrkDocSelectedElement() character
*					setAppVisible(tlVisible) boolean
*					buildHeader() character
*					buildExpFieldsArray() ---
*					getExportFieldName(tcFieldId) character
*					buildDataCursor() ---
*					importFromSpreadSheet() boolean
*					importFromASCII() boolean
*					exportpreprocessing() boolean
*					MAIN() boolean
*					exportASCII() ---
*					exportXmlWebDAVOX([tnTabIdents]) ---
*					exportXml_VisualFoxPro() ---
*					exportDBF_VisualFoxPro() ---
*					exportDBF_FoxPro2X() ---
*					exportXLS_Excel5() ---
*					exportXLS_Excel2() ---
*					exportWK1_Lotus123_V2() ---
*					callHttpForm([tlSend, tcXmlFileLocation]) ---
*					writeLine(tcLine) boolean
*                 	Number2Spread(tnColumn) character
*
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF
PROTECTED cTAB						&& TAB


****************
* SET BY SETUP *
****************
PROTECTED lSetupOk					&& Was the Setup ok?
PROTECTED cImportFile				&& full path+filename of import file

PROTECTED cOutDirName				&& The output directory name
PROTECTED cOutFileName				&& The output filename
PROTECTED oMsgProgressBar			&& ProgressBar
PROTECTED oSegProgressBar			&& ProgressBar
PROTECTED lDebugMode				&& DebugMode


******************
* SET BY SETUPID *
******************
PROTECTED nImpTypeId				&& Id for the ImportType table entry
PROTECTED cImpTypeName				&& Name of the ImportType
PROTECTED nImpFileId				&& Id for the Importfile table entry
PROTECTED nExpTypeId				&& Id for the ExportType table entry
PROTECTED cExpTypeName				&& Name of the ExportType
PROTECTED nExpFileId				&& Id for the Exportfile table entry


****************
*    OTHER     *
****************
PROTECTED nFileHandle				&& The file handle
PROTECTED oApplication				&& Application object


********************************
* SETTINGS IN TABLE EXPORTFILE *
********************************
PROTECTED lAppendMode				&& activate/use append mode and add new content at the bottom of the file
PROTECTED lCRLF_Line				&& end a line with a CRLF?
PROTECTED lCRLF_Item				&& each item ends with CRLF?
PROTECTED lBlankLine				&& add a extra CRLF for a blank line
PROTECTED cSeparator				&& separator string
PROTECTED cKeyItemSep				&& key value separator; Example: 'key = value' where ' = ' is the key value separator
PROTECTED cRepBlock					&& replacement block with internal replaceable fields (<header_elements>, <data_elements>)
PROTECTED lWriteHeader				&& should I write the header to the file?
PROTECTED lQuoteValue				&& use single quote
PROTECTED cEscCharacter				&& escape character to be used
PROTECTED cExpPreProcessingCode		&& SourceCode to be executed after table is filled with data 
									&& (the source will be compiled and executed)
PROTECTED cExpAdditionalParam		&& Memo is used to set additional parameters used later
									&& (e.g. calling WebDAV send form with parameters).


***********************************
* SETTINGS IN TABLE IMPORTFILEDEF *
***********************************
PROTECTED nColAheadCnt				&& read ahead counter for columns
PROTECTED nRowAheadCnt				&& read ahead counter for rows
PROTECTED nDataRow					&& line where the data starts
PROTECTED nHeaderRow				&& line where the header is located
PROTECTED cImpSeparator				&& separator string
PROTECTED cImpEscCharacter			&& escape character to be used

************************************
* SETTINGS IN TABLE EXPORTFIELDDEF *
************************************
DIMENSION aExpFields(1, 2)
PROTECTED nExpFieldsArrayRows		&& number of rows in aExpFields


****************
*    TABLES    *
****************
cTypesSel      = ""
cImpFileSel    = ""
cImpFieldSel   = ""
cExpFileSel    = ""
cExpFieldSel   = ""
cMappingSel    = ""


****************
*    CURSOR    *
****************
cDataCursor = "impDataCur"


*!* Init for this class 
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE init
   DODEFAULT()
   this.cCRLF           = CHR(13)+CHR(10) 
   this.cTAB			= CHR(9)
   
   this.lSetupOk		= .F.
   
   this.cOutFileName	= ""	&& output filename 
   this.cOutDirName		= ""	&& output directory
   this.cImportFile		= ""	&& input file (path+filename)
   this.nFileHandle		= -1	&& The filename handle   
RETURN
ENDPROC


*!* Setup for this class 
*!*
*!* Required Parameter(s):
*!* tcImportFile	= Input filename
*!*	tcOutFile		= Output filename
*!*	tcOutDir		= Output directory name
*!*	
*!* Optional Parameter(s):     
*!*	toMsgProgressBar	= ProgressBar (default is none)
*!*	toSegProgressBar	= ProgressBar (default is none)
*!*	tlDebugMode			= DebugMode (default is .F.)
*!*
PROCEDURE setup
LPARAMETERS tcImportFile, tcOutFile, tcOutDir, toMsgProgressBar, toSegProgressBar, tlDebugMode
LOCAL llok
   llok = .T.
   
  	IF llok AND VARTYPE(tcImportFile) == "C"
		tcImportFile = ALLTRIM(tcImportFile)
		IF EMPTY(tcImportFile) THEN
			llok=.F.
		ELSE
			this.cImportFile = tcImportFile
		ENDIF
	ELSE
		llok = .F.
	ENDIF

  	IF llok AND VARTYPE(tcOutFile) == "C"
		tcOutFile = ALLTRIM(tcOutFile)
		IF EMPTY(tcOutFile) THEN
			llok=.F.
		ELSE
			this.cOutFileName = tcOutFile
		ENDIF
	ELSE
		llok = .F.
	ENDIF
	
	IF llok AND VARTYPE(tcOutDir) == "C"
		tcOutDir = ALLTRIM(tcOutDir)
		IF EMPTY(tcOutDir) THEN
			llok = .F.
		ELSE
			this.cOutDirName = ADDBS(tcOutDir)
		ENDIF
	ELSE
		llok =.F.
	ENDIF	
	
	
	IF llok AND VARTYPE(toMsgProgressBar) == "O"
		this.oMsgProgressBar = toMsgProgressBar&& set MsgProgressBar reference from Parameter
	ENDIF

	IF llok AND VARTYPE(toSegProgressBar) == "O"
		this.oSegProgressBar = toSegProgressBar&& set SegProgressBar reference from Parameter
	ENDIF

	IF llok AND VARTYPE(tlDebugMode) == "L"
		this.lDebugMode = tlDebugMode && set DebugMode
	ELSE
		this.lDebugMode = .F. && as default (no parameter) turn the debugmode off
	ENDIF
   
   this.lSetupOk = llok && set a marker to show if the setup returned ok or not
RETURN(llok)
ENDPROC



*!* Setup ids for this class 
*!*
*!* Required Parameter(s):
*!*	tnImpTypeId		= ImportType id
*!*	tcImpTypeName	= ImportType name
*!*	tnImpFileId		= Importfile id
*!*	tnExpTypeId		= ExportType id
*!*	tcExpTypeName	= ExportType name
*!*	tnExpFileId		= Exportfile id
*!*	
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE setupId
PARAMETERS tnImpTypeId, tcImpTypeName, tnImpFileId, tnExpTypeId, tcExpTypeName, tnExpFileId
LOCAL llOk
	llOk = .T.
	
	IF llOk AND VARTYPE(tnImpTypeId) == "N"
		this.nImpTypeId = tnImpTypeId
	ELSE
		llOk = .F.
	ENDIF

	IF llOk AND VARTYPE(tcImpTypeName) == "C"
		this.cImpTypeName = tcImpTypeName
	ELSE
		llOk = .F.
	ENDIF
	
	IF llOk AND VARTYPE(tnImpFileId) == "N"
		this.nImpFileId = tnImpFileId
	ELSE
		llOk = .F.
	ENDIF
	
	
	
	IF llOk AND VARTYPE(tnExpTypeId) == "N"
		this.nExpTypeId = tnExpTypeId
	ELSE
		llOk = .F.
	ENDIF
	
	IF llOk AND VARTYPE(tcExpTypeName) == "C"
		this.cExpTypeName = tcExpTypeName
	ELSE
		llOk = .F.
	ENDIF
	
	IF llOk AND VARTYPE(tnExpFileId) == "N"
		this.nExpFileId = tnExpFileId
	ELSE
		llOk = .F.
	ENDIF
	
RETURN (llok)
ENDPROC



*!* Open Application
*!* This function opens the application to be used in the following procedures and functions
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION openApplication
LOCAL llok
	llok = .T.
RETURN(llok)
ENDFUNC


*!* Close Application
*!* This function closes and removes the application that has been used
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION closeApplication
LOCAL llok
	llok = .T.
RETURN(llok)
ENDFUNC


*!* Open Application Work Document
*!* This function opens the working document 
*!* The input file can be specified as parameter in this method or at the setup
*!*
*!* Optional Parameter(s):     
*!*	tcImportFile	= The input file that has to be opened and used to import the data from it
*!*
FUNCTION openAppWrkDoc
PARAMETERS tcImportFile
LOCAL llok
	llok = .T.
	IF EMPTY(tcImportFile)
		llok = .F.
	ENDIF
RETURN(llok)
ENDFUNC



*!* Select Column and Row of Application Work Document
*!* This function selects the column and row of the work document
*!*
*!* Required Parameter(s):
*!*	tnColumn	= The column that should be selected
*!* tnRow		= The row that should be selected
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION selectAppWrkDocColumnRow
PARAMETERS tnColumn, tnRow
LOCAL llok
	llok = .F.
	IF (VARTYPE(tnColumn) == "N" AND VARTYPE(tnRow) == "N") THEN
		llok = .T.
	ENDIF
RETURN(llok)
ENDFUNC


*!* Get Selected Element of Application Work Document
*!* This function returns the selected element of the work document
*!*
*!* Required Parameter(s):
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getAppWrkDocSelectedElement
LOCAL lcString
	lcString = ""
RETURN(lcString)
ENDFUNC


*!* Set Application Visible
*!* This function makes the application visible or invisible to the enduser
*!*
*!* Required Parameter(s):
*!* tlVisible	= Visible TRUE (.T.) or FALSE (.F.)
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION setAppVisible
PARAMETERS tlVisible
LOCAL llok
	llok = .F.
	IF VARTYPE(tlVisible) == "L" THEN
		llok = .T.
	ENDIF
RETURN(llok)
ENDFUNC


*!* Builds the export Header string with the selected separator from the export field definition
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION buildHeader
LOCAL llOk, lcOldSel, lcHeader, lnRowSel, lnMaxRows, lnRowCnt
	llOk        = .T.
	lcOldSel    = STR(SELECT(0))
	lcHeader    = ""
	lnRowSel    = 0
	lnRowCnt    = 1
	lnMaxRows   = ALEN(this.aExpFields, 1) && return the number of rows
	
	&& build the string from the columns that can be found in the array aExpField (second column)
	&& the order is defined by the export fieldid column in the cMappingSel table
	SELECT (this.cMappingSel)
	SET ORDER TO EXPSORT
	SCAN FOR IMPFILEID = this.nImpFileId AND EXPFILEID = this.nExpFileId AND EXPCOLUMN>0 AND ACTIVE=.T.
		lnRowSel = ASCAN(this.aExpFields, expColumn, 1, 0, 1, 14) && return row with the found value at column 1 (look for exact match)
		IF (lnRowSel > 0) THEN
			lcHeader = lcHeader + ALLTRIM(TRANSFORM(  this.aExpFields(lnRowSel, 2)  )) + this.cSeparator
		ENDIF
	ENDSCAN
	
	&& remove last separator from the string if the string and the separator are not empty
	IF NOT EMPTY(lcHeader) AND NOT EMPTY(this.cSeparator) THEN 
		lcHeader = SUBSTR(lcHeader, 1, LEN(lcHeader)-1)
	ENDIF
	SELECT &lcOldSel
	RETURN(lcHeader)
ENDFUNC




*!* Builds the aExpFields Array from a JOIN on mapping, importfielddef and exportfielddef 
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE buildExpFieldsArray
LOCAL lcCreateFullMapping
	SELECT columnid, name FROM (this.cExpFieldSel) WHERE ExpFileId = this.nExpFileId INTO ARRAY this.aExpFields
	
	lcCreateFullMapping = "SELECT DISTINCT "+(this.cMappingSel)+".expcolumn, "+(this.cExpFieldSel)+".name expcolname, "+(this.cMappingSel)+".expfileid, "+(this.cMappingSel)+".impcolumn, "+(this.cImpFieldSel)+".name impcolname, "+;
		(this.cMappingSel)+".impfileid"+;
	" FROM "+(this.cImpFieldSel)+" INNER JOIN "+(this.cMappingSel)+;
		" ON "+(this.cImpFieldSel)+".columnid = "+(this.cMappingSel)+".impcolumn AND "+(this.cImpFieldSel)+".impfileid = "+(this.cMappingSel)+".impfileid INNER JOIN "+(this.cExpFieldSel)+;
		" ON "+(this.cMappingSel) +".expcolumn = "+(this.cExpFieldSel)+".columnid AND "+(this.cMappingSel)+".expfileid = "+(this.cExpFieldSel)+".expfileid"+;
	" WHERE active=.T. AND "+;
		(this.cMappingSel)+".impfileid = this.nImpFileId AND "+(this.cMappingSel)+".expfileid = this.nExpFileId"+;
		" ORDER BY "+(this.cMappingSel)+".expColumn ASC"+;
		" INTO ARRAY this.aExpFields"
	
	&& execute build command to insert a joint table into the array this.aExpFields[]
	&lcCreateFullMapping
	
	this.nExpFieldsArrayRows = ALEN(this.aExpFields, 1) && return the number of rows
ENDPROC


*!* GetExportFieldName
*!* This method will return the name for a given columnid if it can be found.
*!*
*!* Required Parameter(s)
*!* tcFieldId =	The field id enclosed in "_". 
*!*				Example:	"_4711_" meaning FieldId = "4711" for which it will return
*!*							the name if it can be found else it returns a blank string.
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getExportFieldName
LPARAMETERS tcFieldId
LOCAL lcExportFieldName, lnArrayIndex, lnFieldId
	lcExportFieldName = ""
	lnArrayIndex      = 0
	lnFieldId         = 0
	
	IF VARTYPE(tcFieldId) = "C" THEN
		lnFieldId = VAL(STRTRAN(tcFieldId, "_", ""))
		IF lnFieldId > 0 THEN
			lnArrayIndex = ASCAN(this.aExpFields, lnFieldId, -1, -1, 1, 8)
			IF lnArrayIndex>0 THEN
				lcExportFieldName = ALLTRIM(this.aExpFields(lnArrayIndex, 2)) && get the ExpFieldName
			ENDIF
		ENDIF
	ENDIF

RETURN(lcExportFieldName)
ENDFUNC


*!* Builds a cursor from the specified output fields that can be found 
*!* in the array this.aExpFields
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE buildDataCursor
LOCAL lnRowCounter, lnMaxRows, lcBuildCursorCommand, lcColumns
	lnRowCounter         = 0
	lnMaxRows            = this.nExpFieldsArrayRows && return the number of rows
	lcBuildCursorCommand = ""
	lcColumns            = ""

	&& TODO: use the ExportFieldDefinition for the type (character,numeric, date,...) of the column
	FOR lnRowCounter = 1 TO lnMaxRows
		lcColumns = lcColumns + "_" + TRANSFORM(this.aExpFields(lnRowCounter, 1)) + "_" + " " + "C(254)," 
	ENDFOR

	IF lnMaxRows>0 THEN
		&& cut of last ","
		lcColumns = SUBSTR(lcColumns, 1, LEN(lcColumns)-1)
		
		&& build the complete cursor creation command
		lcBuildCursorCommand = "CREATE CURSOR (this.cDataCursor) ("+lcColumns+")"
		
		&& execute the cursor creation command
		&lcBuildCursorCommand
	ENDIF
ENDPROC




*!* Import from SpreadSheet
*!* This function imports data from a supported spreadsheet application
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION importFromSpreadSheet
LOCAL llOk, lcOldSel, lcTextField
LOCAL lnColAheadCnt, lnRowAheadCnt
LOCAL lnColCount, lnRowCount, lcTmpHelper, llNewCursorRowAppended, lnExportColumnId

	llok         = .T.
	lcOldSel     = STR(SELECT(0))
	lcTextField  = "" && This element holds every TextElement at the selected position
	lcDataLine   = "" &&
	
	lnColAheadCnt = this.nColAheadCnt
	lnRowAheadCnt = this.nRowAheadCnt
		
	lnColCount = 1 && Start Column
	lnRowCount = 1 && Start Row
	
	lcTmpHelper            = ""		&& Helper for: this.<cTableName> + ".<FieldName>"
	llNewCursorRowAppended = .F.	&& maker to show if a new blank row has been appended to the dataCursor
	lnExportColumnId       = 0		&& ExportColumnId (field expcolumn in mapping table)
	


	IF FILE(this.cImportFile) THEN
		IF this.lDebugMode THEN
			=MESSAGEBOX("Import Datei gefunden...")
		ENDIF
		IF this.openApplication() THEN && open Application
			IF this.lDebugMode THEN
				=MESSAGEBOX("Anwendung wurde geoeffnet...")
			ENDIF
			this.openAppWrkDoc(this.cImportFile) && open the already found file to work with
&&			this.setAppVisible(.T.)
	
			&& data lines
			lnRowCount = this.nDataRow && in which row does the data start
			
			DO WHILE lnRowAheadCnt > 0
				&& select mapping file
				SELECT (this.cMappingSel)
				SET ORDER TO EXPSORT
				SCAN FOR impfileid = this.nImpFileId AND expfileid = this.nExpFileId AND ACTIVE = .T. AND expcolumn > 0 && WHILE lnColAheadCnt > 0 
					lcTmpHelper = this.cMappingSel + ".ImpColumn"
					lnColumnCount = &lcTmpHelper
					
					lcTmpHelper = this.cMappingSel + ".ExpColumn"
					lnExportColumnId = &lcTmpHelper
					
					this.selectAppWrkDocColumnRow(lnColumnCount, lnRowCount)
					lcTextField = this.getAppWrkDocSelectedElement()
					
					&& check element and 
					IF EMPTY(lcTextField) THEN
						lnColAheadCnt = (lnColAheadCnt - 1)
					ELSE && element not empty...work with it
						lnColAheadCnt = this.nColAheadCnt
					
						&& add new row to cursor file if it hasn't been appended yet
						SELECT (this.cDataCursor)
						IF !llNewCursorRowAppended THEN
							APPEND BLANK
							llNewCursorRowAppended = .T.
						ENDIF
						&& TODO: optimize...collect all replaces and execute the replace after each line is completed
						&& e.g.: REPLACE _1_ WITH AELEMENT(laTextField,1,1), _2_ WITH AELEMENT(laTextField,1,2),...
						REPLACE ("_"+TRANSFORM(lnExportColumnId)+"_") WITH lcTextField
												
					ENDIF
					IF lnColAheadCnt == 0 THEN && every column was empty...so now say that the complete row was empty
						lnRowAheadCnt = lnRowAheadCnt - 1
						lnColAheadCnt = this.nColAheadCnt
						EXIT && leave SCAN...ENDSCAN LOOP
					ENDIF
					
				ENDSCAN && COLUMN
				
				lnRowCount = lnRowCount + 1 && next row			
				llNewCursorRowAppended = .F. && reset the marker to show that no new row has been appended to the dataCursor
			ENDDO && ROW
			
			this.closeApplication() && close Application after work is done...
			
		ELSE
			MESSAGEBOX("Anwendung wurde nicht gestartet...")
		ENDIF && end if openExcel executed successfully
	ELSE
		MESSAGEBOX("Importdatei ("+this.cImportFile+") nicht gefunden...")	
	ENDIF && end if cImportFile found
	
	SELECT &lcOldSel
RETURN(llok)
ENDFUNC


*!* Import from ASCII file
*!* This function imports data from a text file
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION importFromASCII
LOCAL llOk, lcOldSel, lcTextField
LOCAL lnColCount, lnRowCount, lcTmpHelper, llNewCursorRowAppended, lnExportColumnId
LOCAL lnFileHandle, lcCompleteLine, lcImpSeparator

	llok         = .T.
	lcOldSel     = STR(SELECT(0))
	lcTextField  = "" && This element holds every TextElement at the selected position
	lcDataLine   = "" &&
	
		
	lnColCount = 1 && Start Column
	lnRowCount = 1 && Start Row
	
	lcTmpHelper            = ""		&& Helper for: this.<cTableName> + ".<FieldName>"
	llNewCursorRowAppended = .F.	&& maker to show if a new blank row has been appended to the dataCursor
	lnExportColumnId       = 0		&& ExportColumnId (field expcolumn in mapping table)
	
	lnFileHandle   = -1						&& set default file handle
	lcCompleteLine = ""						&& set default complete line to empty string
	lcImpSeparator = this.cImpSeparator		&& set the separator character - local variable is faster to access
	
	IF FILE(this.cImportFile) THEN
		IF this.lDebugMode THEN
			=MESSAGEBOX("Import Datei gefunden...")
		ENDIF

		&& use a local filehandle variable (it's faster than the property)
		lnFileHandle = FOPEN(this.cImportFile, 0) && open file as buffered readonly
		
		IF (lnFileHandle != -1) THEN && check filehandle (-1 = failed to open, otherwise file opened successfully)
			IF this.lDebugMode THEN
				=MESSAGEBOX("Datei wurde geffnet...")
			ENDIF
			
			&& data lines
			lnRowCount = this.nDataRow && in which row does the data start
			
			&& Skip lines until lnRowCount reaches 1
			IF (lnRowCount > 1) THEN
				DO WHILE NOT FEOF(lnFileHandle) AND lnRowCount > 1
					FGETS(lnFileHandle, 8192)
					lnRowCount = lnRowCount - 1
				ENDDO
			ENDIF
			
			&& As long as the end of the file is not reached get lines with data...
			DO WHILE NOT FEOF(lnFileHandle)
				&& read the complete line
				lcCompleteLine = FGETS(lnFileHandle, 8192)
				
				&& select mapping file
				SELECT (this.cMappingSel)
				SET ORDER TO EXPSORT
				SCAN FOR impfileid = this.nImpFileId AND expfileid = this.nExpFileId AND ACTIVE = .T. AND expcolumn > 0 && WHILE lnColAheadCnt > 0 
					lcTmpHelper = this.cMappingSel + ".ImpColumn"
					lnColumnCount = &lcTmpHelper
					
					lcTmpHelper = this.cMappingSel + ".ExpColumn"
					lnExportColumnId = &lcTmpHelper
				
					&& Check if the column does exist...if not skip it else process the column
					IF (lnColumnCount <= edi2string(lcCompleteLine, 0, lcImpSeparator)) THEN
						lcTextField = edi2string(lcCompleteLine, lnColumnCount, lcImpSeparator)
						&& element not empty...work with it
						IF NOT EMPTY(lcTextField) THEN							
							&& add new row to cursor file if it hasn't been appended yet
							SELECT (this.cDataCursor)
							IF !llNewCursorRowAppended THEN
								APPEND BLANK
								llNewCursorRowAppended = .T.
							ENDIF
							
							&& TODO: optimize...collect all replaces and execute the replace after each line is completed
							&& e.g.: REPLACE _1_ WITH AELEMENT(laTextField,1,1), _2_ WITH AELEMENT(laTextField,1,2),...
							REPLACE ("_"+TRANSFORM(lnExportColumnId)+"_") WITH lcTextField
												
						ENDIF && end if lcTextField is not empty
						
					ENDIF && end if check if column exists...
				ENDSCAN && COLUMN
				
				lnRowCount = lnRowCount + 1 && next row			
				llNewCursorRowAppended = .F. && reset the marker to show that no new row has been appended to the dataCursor
			ENDDO && ROW
			
			IF NOT FCLOSE(lnFileHandle) THEN && close file after work is done...
				MESSAGEBOX("Der Import wurde durchgefhrt, aber die Datei konnte nicht geschlossen werden!", 16, "Import Fehler") && ERROR sign
			ENDIF
			
		ELSE
			MESSAGEBOX("Die Datei konnte nicht geffnet werden...", 16, "Import Fehler") && ERROR sign
			llok =.F.
		ENDIF && end if openExcel executed successfully
	ELSE
		MESSAGEBOX("Importdatei ("+this.cImportFile+") nicht gefunden...", 16, "Import Fehler") && ERROR sign
		llok = .F.
	ENDIF && end if cImportFile found
	
	SELECT &lcOldSel
RETURN(llok)
ENDFUNC


*!* ExportPreProcessing
*!* This function starts the preprocessing for the export if there is code to process.
*!* It works on the temporary table defined with the export fields and the data put 
*!* there from the import data thought the mapping process .
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED FUNCTION exportPreProcessing
LOCAL llok as Boolean, lcScriptFile
LOCAL loFoxScript as Object, llFoxScript as Boolean

	llok = .T.
	lcScriptFile = "FOXSCRIPT_CONTROL"
	llFoxScript = .F.
	
	&& If there is code for preprocessing start...else skip processing
	IF NOT EMPTY(this.cExpPreProcessingCode)
		&& call methods from foxscript_control class
		loFoxScript = NEWOBJECT(lcScriptFile, lcScriptFile+".prg")
		IF VARTYPE(loFoxScript) == "O" AND NOT ISNULL(loFoxScript) THEN
			IF loFoxScript.setup() THEN
				llFoxScript = .T.
				llok = loFoxScript.assembleUserScriptCode(this.cExpPreProcessingCode)
				IF llok THEN
					llok = loFoxScript.executeUserScriptCode()					
				ENDIF					
						
				&& cleanup program files
				IF (!loFoxScript.cleanUpProgramFiles()) THEN
					llok = llok AND .F.
				ENDIF
					
			ENDIF
		ENDIF
		
		IF !llok THEN
			=MESSAGEBOX("Preprocessing-Fehler:"+this.cCRLF+loFoxScript.getErrorMessages(), 16, "Export preprocessing") && ERROR sign
		ELSE
			IF !llFoxScript THEN
				MESSAGEBOX("Preprocessing-Fehler:"+this.cCRLF+"Das FoxScript Objekt konnte nicht erstellt werden."+this.cCRLF+"Die weitere Verarbeitung wurde abgebrochen!", 16, "Export preprocessing") && ERROR sign
				llok = .F.
			ENDIF
		ENDIF	
	ENDIF
RETURN llok
ENDFUNC



*!* *************************
*!* MAIN METHOD of this class 
*!* *************************
PROCEDURE main
LOCAL llok, lcOldSel
LOCAL lnRowAheadCnt, lnColAheadCnt, lnLastRowData, lnLastColData, lcTmpHelper, lnCountResult
	llok       = .T.
	lcOldSel = STR(SELECT(0))

	lnRowAheadCnt = 0 && read ahead counter of rows (counts down from x to 0)
	lnColAheadCnt = 0 && read ahead counter of columns (counts down from x to 0)
	lnLastRowData = 0 && last row positon with data
	lnLastColData = 0 && last column position with data
	lnCountResult = 0 && result how many rows are in the DataCursor
	
	SELECT (this.cExpFileSel)
	this.lAppendMode   = Append
	this.lCRLF_Line    = CRLF_Line
	this.lCRLF_Item    = CRLF_Item
	this.lBlankLine    = Blank_Line
	this.cSeparator    = EleDivider
	this.cKeyItemSep   = KeyItemSep
	this.cRepBlock     = RepBlock
	this.lWriteHeader  = w_Header
	this.lQuoteValue   = QuoteValue
	this.cEscCharacter = Esc_Char
	this.cExpPreProcessingCode = Code_Src
	this.cExpAdditionalParam = Addi_Param && Addition parameters (KEY:=Value format)
	
	SELECT (this.cImpFileSel)
	this.nColAheadCnt           = r_ColAhead
	this.nRowAheadCnt           = r_RowAhead
	this.nHeaderRow             = HeaderRow
	this.nDataRow               = DataRow
	this.cImpSeparator          = EleDivider
	this.cImpEscCharacter       = Esc_Char
	

	this.buildExpFieldsArray()
	this.buildDataCursor()

	
	IF this.lSetupOk THEN
*!*			&& update progress bar for the main activities
*!*			IF VARTYPE(this.oMsgProgressBar)="O"
*!*				this.oMsgProgressBar.setvals("Zeile "+ALLTRIM(STR(lnMsgCounter))+" von "+ALLTRIM(STR(lnMaxMsg)), lnMsgCounter, lnMaxMsg)
*!*			ENDIF
		
		&& ===========================================================================
		&&								
		&& ===========================================================================
		&& choose the correct import routine/application type
		DO CASE
			&& SPREADSHEET (MS-EXCEL/OpenOffice Calc)
			CASE (ATC("EXCEL", this.cImpTypeName) > 0 OR ATC("OPENOFFICECALC", this.cImpTypeName) > 0)
				this.importFromSpreadSheet()
				
			&& FoxPro XML Cursor
			CASE ATC("XML_VISUALFOXPRO", this.cImpTypeName) > 0
				this.importFromFoxXMLCursor()
				
			&& ASCII file (separator defined in definition table)
			CASE ATC("ASCII", this.cImpTypeName) > 0
				this.importFromASCII()
		ENDCASE
		
		
		&& check if there is data in this.cDataCursor
		SELECT (this.cDataCursor)
		GO TOP
		IF NOT EOF() && yes, we have data
				COUNT ALL TO lnCountResult
				=MESSAGEBOX("Es wurden "+TRANSFORM(lnCountResult)+" Datenstze eingelesen!",64, "Information zum Datenimport") && INFORMATION sign
				
				&& Start preprocessing if code is available (function checks if code available)...
				llok = llok AND this.exportPreProcessing()
				IF llok THEN
					&& Which export type?
					DO CASE
						CASE ATC("ASCII", this.cExpTypeName) > 0
							this.exportASCII()
						
						CASE ATC("XML_VISUALFOXPRO", this.cExpTypeName) > 0
							this.exportXml_VisualFoxPro()

						CASE ATC("DBF_VISUALFOXPRO", this.cExpTypeName) > 0
							this.exportDBF_VisualFoxPro()
													
						CASE ATC("DBF_FOXPRO2X", this.cExpTypeName) > 0
							this.exportDBF_FoxPro2X()
							
						CASE ATC("XLS_EXCEL5+", this.cExpTypeName) > 0
							this.exportXLS_Excel5()
							
						CASE ATC("XLS_EXCEL2", this.cExpTypeName) > 0
							this.exportXLS_Excel2()
							
						CASE ATC("WK1_LOTUS123_V2", this.cExpTypeName) > 0
							this.exportWK1_Lotus123_V2()
							
						CASE ATC("XML_WEBDAV_OX", this.cExpTypeName) > 0
							this.exportXml_WebDAV_OX(1)
							this.callHttpForm(.T.,this.cOutDirName+this.cOutFileName)
					ENDCASE
				ENDIF
			
		ELSE && No, we don't have data
			WAIT WINDOW "DataCursor is empty!" TIMEOUT 5
		ENDIF
		
	ELSE && the setup has not been called or something was wrong
		llok = this.lSetupOk
		=MESSAGEBOX("Es gab einen Fehler im Setup der Kontrollkomponente. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler")
	ENDIF

	SELECT &lcOldSel
RETURN(llok)
ENDPROC


*!* Export the temporary DataCursor into a ASCII file 
*!* (definition selected in EXPORTFILE are used like separator, line-breaks,...)
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportASCII
LOCAL llFileOk
LOCAL llOk, lcOldSel, lcField, lcTextField, lcDataLine, lcHeaderLine
LOCAL lnFieldCount, lcScope
DIMENSION laFields[1]
	llFileOk = .F.
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	lcField      = ""
	lcValue      = ""
	lcDataLine   = ""
	lcHeaderLine = "" && Header line if "<header_elements>" can be found in cRepBlock OR lHeader is TRUE

	lnFieldCount = 0
	lcScope      = "ALL"

	IF this.lAppendMode THEN && append is active
		IF FILE(this.cOutDirName+this.cOutFileName) && check if file exists
			this.nFileHandle = FOPEN(this.cOutDirName+this.cOutFileName, 2) && if yes reopen it
		ELSE
			this.nFileHandle = FCREATE(this.cOutDirName+this.cOutFileName) && else create new file
		ENDIF
	ELSE && no append active...just create a new file
		this.nFileHandle = FCREATE(this.cOutDirName+this.cOutFileName)
	ENDIF
	
	IF (this.nFileHandle != -1) THEN && file could be created...
		FSEEK(this.nFileHandle, 0, 2) && jump to end of file
		
		IF this.lDebugMode THEN
			=MESSAGEBOX("Ausgabedatei erstellt...")
		ENDIF

		&& START WORKING WITH DATA
		&& ===============================================================
		IF this.lWriteHeader OR ATC("<header_elements>",this.cRepBlock)>0  THEN
			lcHeaderLine = this.BuildHeader()
		ENDIF
						

		SELECT (this.cDataCursor)
		lnFieldCount = AFIELDS(laFields)
		
		SCAN &lcScope
			FOR lnCount=1 TO lnFieldCount
				lcField = this.getExportFieldName(laFields[lnCount, 1])
				lcTextField = ALLTRIM(TRANSFORM(EVALUATE(laFields[lnCount, 1])))


				&& TODO: check for strings containing a single quote like "book's" and escape the single quote sign
				IF this.lQuoteValue THEN
					IF OCCURS("'",lcTextField)>0 AND NOT EMPTY(this.cEscCharacter) THEN
						&& TODO: Escape the single quote
					ENDIF
					lcTextField = "'"+lcTextField+"'"
				ENDIF
				
				IF EMPTY(this.cKeyItemSep) THEN && concat the strings 
					IF NOT EMPTY(this.cSeparator)
						lcDataLine = lcDataLine + lcTextField + this.cSeparator
					ENDIF
				ELSE && BUILD KEY = VALUE elements
					&& lcDataLine = this.aExpField[lnColumn]+this.cKeyItemSep+lcTextField
					&& writeLine(lcDataLine)
					&& lcDataLine = "" && delete the previously created/fetched dataline
				ENDIF

			
			ENDFOR
			
			
			&& Row completed...
			IF NOT EMPTY(lcDataLine)
				lcDataLine = SUBSTR(lcDataLine, 1, LEN(lcDataLine)-1) && remove last seperator
				IF NOT EMPTY(this.cRepBlock)
					lcDataLine = STRTRAN(this.cRepBlock, "<data_elements>", lcDataLine)
					lcDataLine = STRTRAN(lcDataLine, "<header_elements>", lcHeaderLine)
				ENDIF
				&& put it somewhere
				this.writeLine(lcDataLine)
			ENDIF
				
			lcDataLine = "" && delete the previously created/fetched dataline
			
		ENDSCAN
		&& ==============================================================
		&& STOP WORKING WITH DATA
		
		IF this.nFileHandle != -1 THEN
			llFileOk = FFLUSH(this.nFileHandle) && Flush the rest from the buffer into the file
			llFileOk = llFileOk AND FCLOSE(this.nFileHandle) && Close the temporary file
			llok = llok AND llFileOk
			IF !llok THEN && ERROR
				=MESSAGEBOX("Fehler: Die Ausgabedatei konnte nicht geschrieben werden!", 16, "Fehler")
			ENDIF
		ENDIF
	ELSE
		=MESSAGEBOX("Die Ausgabedatei konnte nicht erstellt werden!", 16, "Fehler")
	ENDIF
	
	SELECT &lcOldSel
ENDPROC



*!* Export the temporary DataCursor into a OX-XML format
*!*
*!* Required Parameter(s):     
*!*	NONE
*!*
*!* Optional Parameter(s): 
*!* tnTabIdents = use tabs to indent xml parts
*!*
PROCEDURE exportXml_WebDAV_OX
LPARAMETERS tnTabIdents
LOCAL llFileOk
LOCAL llOk, lcOldSel, lcField, lcValue, lcXmlText, lcIdentText, lcDSET, lcDPROP, lcDPROPUPDATE
LOCAL lnFieldCount, lcScope
DIMENSION laFields[1]
	llFileOk = .F.
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	lcField        = ""
	lcValue        = ""
	lcXmlText      = ""
	lcIdentText    = IIF(VARTYPE(tnTabIdents)=="N", REPLICATE(this.cTAB, tnTabIdents), "")
	lcDMULTISTATUS = "D:multistatus"
	lcDSET         = "D:set"
	lcDPROP        = "D:prop"
	lcDPROPUPDATE  = "D:propertyupdate"

	lnFieldCount   = 0
	lcScope        = "ALL"


	&& START WORKING WITH DATA
	&& ===============================================================
	SELECT (this.cDataCursor)
	lnFieldCount = AFIELDS(laFields)
	
	lcXmlText = lcXmlText + "<"+'?xml version="1.0" encoding="UTF-8"?'+">" + this.cCRLF
	lcXmlText = lcXmlText + "<" + lcDMULTISTATUS+" "+'xmlns:D="DAV:">' + this.cCRLF
	lcXmlText = lcXmlText + "<" + lcDPROPUPDATE+" "+'xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">' + this.cCRLF
	SCAN &lcScope
		lcXmlText = lcXmlText + this.cTAB +"<" + lcDSET+ ">" + this.cCRLF
		lcXmlText = lcXmlText + this.cTAB +"<" + lcDPROP+ ">" + this.cCRLF
  
		lcXmlText = lcXmlText + lcIdentText + this.cTAB + "<ox:folder_id>-1</ox:folder_id>" + this.cCRLF

   		FOR lnCount=1 TO lnFieldCount
			lcField = this.getExportFieldName(laFields[lnCount, 1])
			lcValue = ANSI2XML(ALLTRIM(TRANSFORM(EVALUATE(laFields[lnCount, 1]))))
			lcValue = STRTRAN(lcValue, CHR(11), CHR(13)+CHR(10)) && replace 'Vertical TAB' (ASCII Code 11=0x0B) with 'CR+LF'
			
			IF EMPTY(lcValue) THEN && ignore empty entries and start next for-loop
				LOOP
			ENDIF
			
			lcXmlText =	lcXmlText + lcIdentText + this.cTAB + ;
						"<" + lcField + ">" + lcValue + "</" + lcField + ">" + this.cCRLF
		ENDFOR

&&		lcXmlText = lcXmlText + lcIdentText + this.cTAB + "<ox:client_id>0</ox:client_id>" + this.cCRLF
		
		lcXmlText = lcXmlText + this.cTAB + "</" + lcDPROP+ ">" + this.cCRLF
		lcXmlText = lcXmlText + this.cTAB + "</" + lcDSET+ ">" + this.cCRLF
	ENDSCAN
	
	lcXmlText = lcXmlText + "</" + lcDPROPUPDATE + ">" + this.cCRLF
	lcXmlText = lcXmlText + "</" + lcDMULTISTATUS + ">" + this.cCRLF
	
	&& ==============================================================
	&& STOP WORKING WITH DATA
	
	&& save the string if not empty
	IF NOT EMPTY(lcXmlText) THEN
		STRTOFILE(lcXmlText, this.cOutDirName+this.cOutFileName)
	ENDIF

	SELECT &lcOldSel
ENDPROC



*!* Export the temporary DataCursor into a Visual FoxPro XML format
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportXml_VisualFoxPro
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	CURSORTOXML(this.cDataCursor, this.cOutDirName+this.cOutFileName, 1, 562, 0, "")
&&	CURSORTOXML(this.cDataCursor, this.cOutDirName+this.cOutFileName, 1, 514, 0, "")
&&	CURSORTOXML(this.cDataCursor, this.cOutDirName+this.cOutFileName, 1, 512, 0, "")
	
	SELECT &lcOldSel
ENDPROC


*!* Export the temporary DataCursor into a Visual FoxPro DBF format
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportDBF_VisualFoxPro
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	SELECT (this.cDataCursor)
	COPY TO (this.cOutDirName+this.cOutFileName) ALL
		
	SELECT &lcOldSel
ENDPROC


*!* Export the temporary DataCursor into the old DOS/MAC/WIN FoxPro DBF format
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportDBF_FoxPro2X
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	SELECT (this.cDataCursor)
	COPY TO (this.cOutDirName+this.cOutFileName) ALL TYPE FOX2X
		
	SELECT &lcOldSel
ENDPROC


*!* Export the temporary DataCursor into a Excel 5 XLS format
*!* Export maximum: 65,535 rows (see the following note)
*!*
*!* NOTE:
*!* =====
*!* Though you can export a maximum of 65,535 rows, which includes one 
*!* row reserved for the field header, versions of Excel earlier than 
*!* 8.0 (Excel 97) display only the first 16,384 rows and cannot import
*!* files containing more than 32,767 rows.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportXLS_Excel5
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	SELECT (this.cDataCursor)
	COPY TO (this.cOutDirName+this.cOutFileName) ALL TYPE XL5
		
	SELECT &lcOldSel
ENDPROC


*!* Export the temporary DataCursor into a Excel 2 XLS format
*!* Export maximum: 65,535 rows (see the following note)
*!*
*!* NOTE:
*!* =====
*!* Though you can export a maximum of 65,535 rows, which includes one 
*!* row reserved for the field header, versions of Excel earlier than 
*!* 8.0 (Excel 97) display only the first 16,384 rows and cannot import
*!* files containing more than 32,767 rows.
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportXLS_Excel2
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	SELECT (this.cDataCursor)
	COPY TO (this.cOutDirName+this.cOutFileName) ALL TYPE XLS
		
	SELECT &lcOldSel
ENDPROC


*!* Export the temporary DataCursor into a Lotus 1-2-3 v2.x WK1 format
*!*
*!* Required Parameter(s): 
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE exportWK1_Lotus123_V2
LOCAL llOk, lcOldSel
	llOk     = .T.
	lcOldSel = STR(SELECT(0))
	
	SELECT (this.cDataCursor)
	COPY TO (this.cOutDirName+this.cOutFileName) ALL TYPE WK1
		
	SELECT &lcOldSel
ENDPROC



*!* Call the Http Send/Receive Form
*!*
*!* Required Parameter(s):     
*!*	NONE
*!*
*!* Optional Parameter(s): 
*!* tlSend            = open the http form as "Send" (Parameter=TRUE - DEFAULT!!) 
*!*					    or as "Receive" (PARAMETER=FALSE)
*!* tcXmlFileLocation = full path of the file to send or where the received file should be saved
*!*
PROCEDURE callHttpForm
LPARAMETERS tlSend, tcXmlFileLocation, tlTransferAll, tlUsrAndPwdRequired
LOCAL llSend, lcConfig, lcKeyValue, lcServerURI, lcPathOnServer
	llSend = .T.
	lcServerURI = ""
	lcPathOnServer = ""
	
	IF NOT EMPTY(this.cExpAdditionalParam)
		lcConfig = this.cExpAdditionalParam
		lcConfig = STRTRAN(lcConfig, CHR(13)+CHR(10), ";")
		FOR lnCounter = 1 TO edi2string(lcConfig, 0, ";")
			lcKeyValue = edi2string(lcConfig, lnCounter, ";")
			IF NOT EMPTY(lcKeyValue) AND edi2string(lcKeyValue, 0, ":=") == 2 THEN
				DO CASE
					CASE UPPER(edi2string(lcKeyValue, 1, ":=")) == "SERVER_URI"
						lcServerURI = edi2string(lcKeyValue, 2, ":=")

					CASE UPPER(edi2string(lcKeyValue, 1, ":=")) == "PATH_ON_SERVER"
						lcPathOnServer = edi2string(lcKeyValue, 2, ":=")
				ENDCASE
			ENDIF
		ENDFOR
	ENDIF
	
	IF VARTYPE(tlSend) == "L" THEN && if logical use the value in tlSend otherwise use the default llSend value
		llSend = tlSend
	ENDIF
	
	IF VARTYPE(tcXmlFileLocation) == "C" THEN
		DO FORM frm0002 WITH llSend, tcXmlFileLocation, .F., .T., lcServerURI, lcPathOnServer
	ELSE
		DO FORM frm0002 WITH llSend
	ENDIF
ENDPROC



*!* WriteLine writes the string submitted as parameter into the open file.
*!* It returns if the write has been successfull or not.
*!*
*!* Required Parameter(s): 
*!* tcLine = Line to write into the file
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION writeLine
PARAMETERS tcLine
LOCAL llok
	llok = .T.
	IF this.nFileHandle != -1 THEN
		IF this.lBlankLine THEN && Add an extra CRLF to provide a blank line
			tcLine = tcLine + this.cCRLF
		ENDIF
		
		IF this.lCRLF_Line THEN  && each line ends with a CRLF...
			IF 0 == FPUTS(this.nFileHandle, tcLine) THEN && write to the file with CR and LF
				llok = .F.
			ENDIF
		ELSE && just put everything in one line...
			IF 0 = FWRITE(this.nFileHandle, tcLine) THEN && write to the file without CR and LF
				llok = .F.
			ENDIF
		ENDIF
	ELSE && no file opened
		llok = .F.
	ENDIF
RETURN(llok)
ENDFUNC


*!* Create calc fields: A1, B1,...BB20
*!* This method creates the column names used in Spreadsheets like EXCEL and OpenOffice Calc
*!*
*!* Required Parameter(s): 
*!* tnColumn = Column number
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION Number2Spread
PARAMETERS tnColumn
LOCAL lcReturn, laArray, lnCounter, lnRest, lnColumn
	lcReturn = ""
	
	IF tnColumn > 0 AND VARTYPE(tnColumn) == "N" THEN
		lnRest = 0
		lnColumn = tnColumn
		DIMENSION laArray[26]
		FOR lnCounter = 1 TO 26 && fill Array with Characters from 'A'..'Z'
			STORE CHR(64+lnCounter) TO laArray[lnCounter] 
		ENDFOR
		DO WHILE lnColumn > 26
			lnRest = lnColumn % 26
			lnColumn = lnColumn / 26
			lcReturn = laArray[lnRest] + lcReturn
		ENDDO
		lcReturn = laArray[lnColumn] + lcReturn
	ENDIF
RETURN(lcReturn)


ENDDEFINE && END OF CLASS DEFINITION
