DEFINE CLASS HTTP_MULTISTATUS_CONTROL AS Custom
* -------------------------------------------------------------------------------- *
*	HTTP_MULTISTATUS_CONTROL
*	========================
*	Desc (short):	HTTP_MultiStatus_Control implements the default handling for
*					http response codes 207 (multistatus).
*					The MultiStatus response is a XML message received from the 
*					HTTP-Server!
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.03 - 2006-01-02
*	VFP Version.:	VFP 9.0
*	Description.:	This class allows to call the basic import data methods
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-12-30 - J.U. -	work started
*						2006-01-01 - J.U. - first tested and working version
*						2006-01-02 - J.U. - completed method list and method description
*													
*	Methods.....:	init() ---
*					setup(tcMultiStatusMessage, [tlDebugMode]) boolean
*					buildMultiStatusCursor() ---
*					analyseMultiStatusResponse() boolean
*					getTotalMSEntries() this.nTotalMSEntries
*					countTotalMSEntries() lnEntries
*					getSimpleMSResult() this.cSimpleMSResult
*					generateSimpleMSResult() lcSimpleResult
*					checkStatusCodes(tcResultCode) ---
*					getMSStatus() this.lMSStatus
*					clearObject() ---
*
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF


****************
* SET BY SETUP *
****************
PROTECTED lSetupOk					&& Was the Setup ok?
PROTECTED cMultiStatus				&& MultiStatus message

PROTECTED lDebugMode				&& DebugMode


****************
*    OTHER     *
****************
PROTECTED oXMLDomDoc				&& XML DOM Document
PROTECTED nTotalMSEntries			&& Total number of all Elements in the MultiStatus message
PROTECTED cSimpleMSResult			&& String (multiline) in form of "<NumberOfElements>x <responsedescription> (Code: <status>)"
PROTECTED lMSStatus					&& Does the result only contain positive MultiStatus results (Code "200" or "HTTP/1.1 200")?
DIMENSION aGroupedResult(1, 3)


****************
*    CURSOR    *
****************
cMSCursor = "msDataCur"



*!* Init for this class 
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE init
	DODEFAULT()
	this.cCRLF		= CHR(13)+CHR(10) 
	this.lSetupOk	= .F.
	this.lMSStatus  = .T.
RETURN
ENDPROC


*!* Setup for this class 
*!*
*!* Required Parameter(s):
*!* tcMultiStatusMessage	= Input MultiStatus message
*!*	
*!* Optional Parameter(s):     
*!*	tlDebugMode				= DebugMode (default is .F.)
*!*
PROCEDURE setup
LPARAMETERS tcMultiStatusMessage, tlDebugMode
LOCAL llOk
	llOk = .T.
	
	IF VARTYPE(tcMultiStatusMessage) == "C" THEN
		IF NOT EMPTY(tcMultiStatusMessage) THEN
			this.cMultiStatus = tcMultiStatusMessage
		ELSE
			llOk = .F.
		ENDIF
	ELSE
		llOk = .F.
	ENDIF
	
	IF llOk THEN 
		this.oXMLDomDoc	= CREATEOBJECT("MSXML2.DOMDocument.4.0")
		IF VARTYPE(this.oXMLDomDoc) == "O" THEN
			this.oXMLDomDoc.async = .F.
			
			&& Activate new parser
			&& parser which offers greater performance, but does not yet support 
			&& asynchronous mode or DTD validation.
			&& Using the new parser can realize a parsing performance improvement 
			&& of between 200 and 400% for XSLT transformations.
			this.oXMLDomDoc.setProperty("NewParser", .T.) 
		ELSE
			llOk = .F.
		ENDIF
	ENDIF
	
	IF llok AND VARTYPE(tlDebugMode) == "L"
		this.lDebugMode = tlDebugMode && set DebugMode
	ELSE
		this.lDebugMode = .F. && as default (no parameter) turn the debugmode off
	ENDIF
	
	this.lSetupOk = llOk && set a marker to show if the setup returned ok or not
RETURN(llok)
ENDPROC



*!* Builds a cursor for the specified fields
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE buildMultiStatusCursor
LOCAL lcBuildCursorCommand
	lcBuildCursorCommand = ""
	
	&& build the complete cursor creation command
	lcBuildCursorCommand = "CREATE CURSOR (this.cMSCursor) (ObjectID C(254), Href C(254), HTTPDesc C(254), HTTPCode C(254))"
		
	&& execute the cursor creation command
	&lcBuildCursorCommand
ENDPROC



*!* ****************************************
*!* AnalyseMultiStatusResponse of this class 
*!* ****************************************
FUNCTION analyseMultiStatusResponse
LOCAL llOk, lnOldSel
LOCAL lnCountResult, lcHref
LOCAL loNode, loNodes, loChildNode, loChildNodes
	llOk			= .T.
	lnOldSel		= STR(SELECT(0))
	lnCountResult	= 0
	lcHref			= ""

	IF llOk AND this.lSetupOk THEN
		&& set default lMSStatus to true - VERY IMPORTANT!!!
		this.lMSStatus  = .T.
		
		&& create cursor for XML parser results
		this.buildMultiStatusCursor()
		
		&& load XML MultiStatus into DOM Parser
		IF NOT(this.oXMLDomDoc.loadXML(this.cMultiStatus))
			llOk = .F.
			=MESSAGEBOX("Es ist ein Fehler beim Laden der MultiStatus-Nachricht aufgetreten. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler")
		ENDIF
		
		&& process XML Document
		IF llOk THEN
			this.oXMLDomDoc.setProperty("SelectionNamespaces", "xmlns:D='DAV:'")
			
			loNodes = this.oXMLDomDoc.selectNodes("/D:multistatus/D:response")
			IF VARTYPE(loNodes) == "O" THEN
				SELECT (this.cMSCursor)
				
				FOR EACH loNode IN loNodes
					IF loNode.hasChildNodes THEN
						APPEND BLANK
						SCATTER MEMVAR

						IF VARTYPE(loNode.selectSingleNode("D:href")) == "O" THEN
							m.Href = loNode.selectSingleNode("D:href").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:status")) == "O" THEN
							m.HTTPCode = loNode.selectSingleNode("D:propstat/D:status").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:responsedescription")) == "O" THEN
							m.HTTPDesc = loNode.selectSingleNode("D:propstat/D:responsedescription").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop")) == "O" THEN
							m.ObjectID = loNode.selectSingleNode("D:propstat/D:prop").nodeTypedValue
						ENDIF
						
						GATHER MEMVAR
					ENDIF && end if loNode.hasChildNodes
				ENDFOR && end foreach loNode IN loNodes
				
				this.nTotalMSEntries = this.countTotalMSEntries()
				this.cSimpleMSResult = this.generateSimpleMSResult()
			ENDIF && check if loNodes is an object
		ENDIF && end if llOk...
		
	ELSE && the setup has not been called or something was wrong
		llok = this.lSetupOk
		=MESSAGEBOX("Es gab einen Fehler im Setup der MultiStatus-Komponente. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler")
	ENDIF

	SELECT &lnOldSel
RETURN(llok)
ENDFUNC



*!* Returns the number of all responses in the multistatus XML message.
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getTotalMSEntries
	RETURN this.nTotalMSEntries
ENDFUNC



*!* Counts all responses in the multistatus XML message and returns the value.
*!* The value is saved in this.nTotalMSEntries (see getTotalMSEntries()).
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED FUNCTION countTotalMSEntries
LOCAL lnOldSel, lnEntries
	lnOldSel	= STR(SELECT(0))
	lnEntries	= 0
	
	&& check if there is data in this.cDataCursor
	SELECT (this.cMSCursor)
	GO TOP
	IF NOT EOF() && yes, we have data
		COUNT ALL TO lnEntries
	ENDIF
	SELECT &lnOldSel
RETURN(lnEntries)
ENDFUNC



*!* Returns a simple String which lists the results in a compact form
*!* (see generateSimpleMSResult()).
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getSimpleMSResult
	RETURN(this.cSimpleMSResult)
ENDFUNC



*!* Create the compact String which lists the results (multiline if more than one different result).
*!* The value is saved in this.cSimpleMSResult (see getSimpleMSResult()).
*!* The result: "<numberOfEntries> x <http-Description> (<http-Statuscode>)"
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED FUNCTION generateSimpleMSResult
LOCAL lnOldSel, lcSimpleResult, lnMaxRows, lnArrayCounter
	lnOldSel		= STR(SELECT(0))
	lcSimpleResult	= ""
	lnMaxRows		= 0
	lnArrayCounter	= 0
	
	SELECT (this.cMSCursor)
	GO TOP
	IF NOT EOF() && yes, we have data
		SELECT COUNT(*) AS NumOfEntries, HttpDesc, HttpCode FROM (this.cMSCursor) GROUP BY HttpCode, HttpDesc ORDER BY HttpCode ASC INTO ARRAY aGroupedResult
		lnMaxRows = ALEN(this.aGroupedResult, 1) && return the number of rows
		FOR lnArrayCounter = 1 TO lnMaxRows
			this.checkStatusCodes(ALLTRIM(aGroupedResult(lnArrayCounter, 3)))
			lcSimpleResult = ALLTRIM(lcSimpleResult + TRANSFORM(aGroupedResult(lnArrayCounter, 1)) + " x " + ALLTRIM(aGroupedResult(lnArrayCounter, 2)) + " (Code: " + ALLTRIM(aGroupedResult(lnArrayCounter, 3)) + ")" + this.cCRLF)
		ENDFOR
	ENDIF
	SELECT &lnOldSel
RETURN(lcSimpleResult)
ENDFUNC



*!* Returns the global MultiStatus variable that will indicate if one or more
*!* errors have occured (see checkStatusCode()).
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getMSStatus
	RETURN(this.lMSStatus)
ENDFUNC



*!* This function checks ever returncode in the MultiStatus message and sets the global
*!* MultiStatus value (this.lMSStatus; see also getMSStatus()).
*!* It's most likely that this function needs to be updated with other possible
*!* ResultCodes as only the compatibility with Open-Xchange returncodes has been checked!
*!*
*!* Required Parameter(s)
*!* tcResultCode	= Single Status Code to check 
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED FUNCTION checkStatusCodes
LPARAMETERS tcResultCode
	IF VARTYPE(tcResultCode) == "C" THEN
		this.lMSStatus = this.lMSStatus AND (AT("HTTP/1.1 200", UPPER(tcResultCode)) > 0 OR AT("200", tcResultCode) > 0)
	ENDIF
ENDFUNC



*!* This method clears the object (e.g. delete the cursor)
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE clearObject
LOCAL lnOldSel
	lnOldSel		= STR(SELECT(0))
	
	SELECT (this.cMSCursor)
	USE
	
	SELECT &lnOldSel
ENDPROC	

ENDDEFINE && END OF CLASS DEFINITION
