DEFINE CLASS OX_STYLESHEETS AS Custom
* -------------------------------------------------------------------------------- *
*	OX_STYLESHEETS
*	=============================================
*	Desc (short):	A set of Stylesheets that allow to upload previously 
*					saved/downloaded Open-Xchange XML files again to the OX server.
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.05 - 2005-08-19
*	VFP Version.:	VFP 7.0
*	Description.:	This class allows to transform Open-Xchange XML files. After 
*					this transformation these files can be uploaded to the 
*					Open-Xchange server.
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-07-31 - J.U. -	work started/initial version ready
*						2005-08-18 - J.U. - added stylesheet and functions to delete calendar
*						2005-08-19 - J.U. - added stylesheets and functions to delete contacts and tasks
*													
*	Methods.....:	init() ---
*					MsXmlTransform(tcXML, tcStyleSheet) returns transformed XML as string
*
*					transformContacts(tcXMLString) returns transformed XML as string
*					setXSLContacts() ---
*					transformDeleteContacts(tcXMLString) returns transformed XML as string
*					setXSLDeleteContacts() ---
*
*					transformCalendar(tcXMLString) returns transformed XML as string
*					setXSLCalendar() ---
*					transformDeleteCalendar(tcXMLString) returns transformed XML as string
*					setXSLDeleteCalendar() ---
*
*					transformTasks(tcXMLString) returns transformed XML as string
*					setXSLTasks() ---
*					transformDeleteTasks(tcXMLString) returns transformed XML as string
*					setXSLDeleteTasks() ---
*                 	                                                               
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF
PROTECTED cTAB						&& TAB


PROTECTED cXSLContacts
PROTECTED cXSLDeleteContacts
PROTECTED cXSLCalendar
PROTECTED cXSLDeleteCalendar
PROTECTED cXSLTasks
PROTECTED cXSLDeleteTasks


*!* Init for this class 
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROTECTED PROCEDURE init
	DODEFAULT()
	this.cCRLF          = CHR(13)+CHR(10)
	this.cTAB			= CHR(9)
	
	this.setXSLContacts()
	this.setXSLDeleteContacts()
	this.setXSLCalendar()
	this.setXSLDeleteCalendar()
	this.setXSLTasks()
	this.setXSLDeleteTasks()
RETURN
ENDPROC

PROTECTED FUNCTION MsXmlTransform
LPARAMETERS tcXML, tcStyleSheet
LOCAL lcXMLReturnString, loXML, loXSLT
	lcXMLReturnString = ""
	IF VARTYPE(tcXML)=="C" AND VARTYPE(tcStyleSheet)=="C" AND NOT EMPTY(tcXML) AND NOT EMPTY(tcStyleSheet) THEN
		&& Create MSXML DOM Document Object
		loXML = CREATEOBJECT("Msxml2.DOMDocument.4.0")
		loXML.validateOnParse = .T.
		loXML.Async = .F.

		&& Load XML data
		loXML.LoadXml(tcXml) && to load data from a file use the Load(Path) method

		&& Create another MSXML DOM Document Object
		loXSLT = CREATEOBJECT("Msxml2.DOMDocument.4.0")
		loXSLT.validateOnParse = .T.
		loXSLT.Async = .F.
		
		&& Load XSLT data
		loXSLT.LoadXml(tcStyleSheet)
&&		loXML.setProperty("encoding", "UTF-8")
		
		&& Transform and return new XML data
		lcXMLReturnString = loXML.transformNode(loXSLT)
		IF loXml.parseError.errorCode != 0 THEN
			=MESSAGEBOX("Error while transforming XML file!",16)
		ENDIF
	
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC


FUNCTION transformContacts
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLContacts)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC


PROTECTED PROCEDURE setXSLContacts
	this.cXSLContacts = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="no" media-type="text/xml"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()!=' + "'" + 'ox:creation_time' + "'" + '">' + this.cCRLF+;
						'						<xsl:if test="name()!=' + "'" + 'ox:object_status' + "'" + '">' + this.cCRLF+;
						'							<xsl:if test="name()!=' +"'"+ 'ox:created_by' + "'" + '">' + this.cCRLF+;
						'								<xsl:if test="name()!=' + "'" + 'ox:last_modified' + "'" + '">' + this.cCRLF+;
						'									<xsl:if test="name()!=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'										<xsl:if test="name()!=' + "'" + 'ox:distributionlist_flag' + "'" + '">' + this.cCRLF+;
						'											<xsl:if test="name()!=' + "'" + 'ox:modified_by' + "'" + '">' + this.cCRLF+;
						'												<xsl:if test="name()!=' + "'" + 'ox:attachment_flag' + "'" + '">' + this.cCRLF+;
						'													<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'												</xsl:if>'+this.cCRLF+;
						'											</xsl:if>'+this.cCRLF+;
						'										</xsl:if>'+this.cCRLF+;
						'									</xsl:if>'+this.cCRLF+;
						'								</xsl:if>'+this.cCRLF+;
						'							</xsl:if>'+this.cCRLF+;
						'						</xsl:if>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLContacts


FUNCTION transformDeleteContacts
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLDeleteContacts)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC

PROTECTED PROCEDURE setXSLDeleteContacts
	this.cXSLDeleteContacts = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<ox:method>DELETE</ox:method>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'						<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLDeleteContacts


FUNCTION transformCalendar
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLCalendar)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC


PROTECTED PROCEDURE setXSLCalendar
	this.cXSLCalendar = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()!=' + "'" + 'ox:creation_time' + "'" + '">' + this.cCRLF+;
						'						<xsl:if test="name()!=' + "'" + 'ox:object_status' + "'" + '">' + this.cCRLF+;
						'							<xsl:if test="name()!=' +"'"+ 'ox:created_by' + "'" + '">' + this.cCRLF+;
						'								<xsl:if test="name()!=' + "'" + 'ox:last_modified' + "'" + '">' + this.cCRLF+;
						'									<xsl:if test="name()!=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'										<xsl:if test="name()!=' + "'" + 'ox:modified_by' + "'" + '">' + this.cCRLF+;
						'											<xsl:if test="name()!=' + "'" + 'ox:private' + "'" + '">' + this.cCRLF+;
						'												<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'											</xsl:if>'+this.cCRLF+;
						'										</xsl:if>'+this.cCRLF+;
						'									</xsl:if>'+this.cCRLF+;
						'								</xsl:if>'+this.cCRLF+;
						'							</xsl:if>'+this.cCRLF+;
						'						</xsl:if>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLCalendar

FUNCTION transformDeleteCalendar
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLDeleteCalendar)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC

PROTECTED PROCEDURE setXSLDeleteCalendar
	this.cXSLDeleteCalendar = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<ox:method>DELETE</ox:method>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'						<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLDeleteCalendar


FUNCTION transformTasks
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLTasks)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC


PROTECTED PROCEDURE setXSLTasks
	this.cXSLTasks = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()!=' + "'" + 'ox:creation_time' + "'" + '">' + this.cCRLF+;
						'						<xsl:if test="name()!=' + "'" + 'ox:object_status' + "'" + '">' + this.cCRLF+;
						'							<xsl:if test="name()!=' +"'"+ 'ox:created_by' + "'" + '">' + this.cCRLF+;
						'								<xsl:if test="name()!=' + "'" + 'ox:last_modified' + "'" + '">' + this.cCRLF+;
						'									<xsl:if test="name()!=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'										<xsl:if test="name()!=' + "'" + 'ox:modified_by' + "'" + '">' + this.cCRLF+;
						'											<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'										</xsl:if>'+this.cCRLF+;
						'									</xsl:if>'+this.cCRLF+;
						'								</xsl:if>'+this.cCRLF+;
						'							</xsl:if>'+this.cCRLF+;
						'						</xsl:if>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLTask

FUNCTION transformDeleteTasks
LPARAMETERS tcXMLString
LOCAL lcXMLReturnString
	lcXMLReturnString = ""
	IF VARTYPE(tcXMLString) == "C" THEN
		lcXmlReturnString = this.MsXmlTransform(tcXmlString, this.cXSLDeleteTasks)
	ENDIF
	RETURN(lcXMLReturnString)
ENDFUNC

PROTECTED PROCEDURE setXSLDeleteTasks
	this.cXSLDeleteTasks = '<?xml version="1.0" encoding="ISO-8859-15"?>'+this.cCRLF+;
						'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>'+this.cCRLF+;
						'	'+this.cCRLF+;
						'	<xsl:template match="/">'+this.cCRLF+;
						'		<D:multistatus xmlns:D="DAV:">'+this.cCRLF+;
						'			<D:propertyupdate xmlns:D="DAV:" xmlns:ox="http://www.open-xchange.org">'+this.cCRLF+;
						'				<xsl:apply-templates select="//D:prop"/>'+this.cCRLF+;
						'			</D:propertyupdate>'+this.cCRLF+;
						'		</D:multistatus>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						''+this.cCRLF+;
						'	<xsl:template match="//D:prop">'+this.cCRLF+;
						'		<D:set>'+this.cCRLF+;
						'			<D:prop>'+this.cCRLF+;
						'				<ox:method>DELETE</ox:method>'+this.cCRLF+;
						'				<xsl:for-each select="child::*">'+this.cCRLF+;
						'					<xsl:if test="name()=' + "'" + 'ox:object_id' + "'" + '">' + this.cCRLF+;
						'						<xsl:copy-of select="current()"/>'+this.cCRLF+;
						'					</xsl:if>'+this.cCRLF+;
						'				</xsl:for-each>'+this.cCRLF+;
						'			</D:prop>'+this.cCRLF+;
						'		</D:set>'+this.cCRLF+;
						'	</xsl:template>'+this.cCRLF+;
						'</xsl:stylesheet>'
ENDPROC && setXSLDeleteTasks

ENDDEFINE && END OF CLASS DEFINITION
