DEFINE CLASS OX_WEBDAV_FOLDERS AS Custom
* -------------------------------------------------------------------------------- *
*	OX_WEBDAV_FOLDERS
*	=============================================
*	Desc (short):	A set of Stylesheets that allow to upload previously 
*					saved/downloaded Open-Xchange XML files again to the OX server.
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.02 - 2006-01-02
*	VFP Version.:	VFP 7.0
*	Description.:	This class allows to transform Open-Xchange XML files. After 
*					this transformation these files can be uploaded to the 
*					Open-Xchange server.
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-09-29 - J.U. -	work started/code skeleton
*						2006-01-02 - J.U. - first working version
*						2006-01-03 - J.U. - minor changes in cursor definition
*						
*	Methods.....:	init() ---
*					setup(tcMultiStatusMessage, [tlDebugMode]) boolean
*					buildMultiStatusCursor() ---
*					analyseMultiStatusResponse() boolean
*					getTotalMSEntries() this.nTotalMSEntries
*					countTotalMSEntries() lnEntries
*					getSimpleMSResult() this.cSimpleMSResult
*					generateSimpleMSResult() lcSimpleResult
*					checkStatusCodes(tcResultCode) ---
*					getMSStatus() this.lMSStatus
*					clearObject() ---
*
* -------------------------------------------------------------------------------- *

****************
* SET BY INIT  *
****************
PROTECTED cCRLF						&& CRLF


****************
* SET BY SETUP *
****************
PROTECTED lSetupOk					&& Was the Setup ok?
PROTECTED cMultiStatus				&& MultiStatus message
PROTECTED lDebugMode				&& DebugMode


****************
*    OTHER     *
****************
PROTECTED oXMLDomDoc				&& XML DOM Document
PROTECTED nTotalMSEntries			&& Total number of all Elements in the MultiStatus message
PROTECTED cSimpleMSResult			&& String (multiline) in form of "<NumberOfElements>x <responsedescription> (Code: <status>)"
PROTECTED lMSStatus					&& Does the result only contain positive MultiStatus results (Code "200" or "HTTP/1.1 200")?
&&DIMENSION aGroupedResult(1, 3)


****************
*    CURSOR    *
****************
cFolderCursor = "FolderCur"



*!* Init for this class 
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE init
	DODEFAULT()
	this.cCRLF		= CHR(13)+CHR(10) 
	this.lSetupOk	= .F.
	this.lMSStatus  = .T.
RETURN
ENDPROC


*!* Setup for this class 
*!*
*!* Required Parameter(s):
*!* tcMultiStatusMessage	= Input MultiStatus message
*!*	
*!* Optional Parameter(s):     
*!*	tlDebugMode				= DebugMode (default is .F.)
*!*
PROCEDURE setup
LPARAMETERS tcMultiStatusMessage, tlDebugMode
LOCAL llOk
	llOk = .T.
	
	IF VARTYPE(tcMultiStatusMessage) == "C" THEN
		IF NOT EMPTY(tcMultiStatusMessage) THEN
			this.cMultiStatus = tcMultiStatusMessage
		ELSE
			llOk = .F.
		ENDIF
	ELSE
		llOk = .F.
	ENDIF
	
	IF llOk THEN 
		this.oXMLDomDoc	= CREATEOBJECT("MSXML2.DOMDocument.4.0")
		IF VARTYPE(this.oXMLDomDoc) == "O" THEN
			this.oXMLDomDoc.async = .F.
			
			&& Activate new parser
			&& parser which offers greater performance, but does not yet support 
			&& asynchronous mode or DTD validation.
			&& Using the new parser can realize a parsing performance improvement 
			&& of between 200 and 400% for XSLT transformations.
			this.oXMLDomDoc.setProperty("NewParser", .T.) 
		ELSE
			llOk = .F.
		ENDIF
	ENDIF
	
	IF llok AND VARTYPE(tlDebugMode) == "L"
		this.lDebugMode = tlDebugMode && set DebugMode
	ELSE
		this.lDebugMode = .F. && as default (no parameter) turn the debugmode off
	ENDIF
	
	this.lSetupOk = llOk && set a marker to show if the setup returned ok or not
RETURN(llok)
ENDPROC



*!* Builds a cursor for the specified fields
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE buildMultiStatusCursor
LOCAL lcBuildCursorCommand
	lcBuildCursorCommand = ""
	
	&& build the complete cursor creation command
	lcBuildCursorCommand = "CREATE CURSOR (this.cFolderCursor) (Href C(254), HTTPDesc C(254), HTTPCode C(254),";
							+" createdby C(254), modifiedby C(254), owner C(254),";
							+" creator C(254), object_id C(254), folder_id C(254),";
							+" module C(254), type C(254), title C(254),";
							+" deffolder L(1), transfer L(1))"
		
	&& execute the cursor creation command
	&lcBuildCursorCommand
ENDPROC



*!* ****************************************
*!*    AnalyseMultiStatusResponse (main)
*!* ****************************************
FUNCTION analyseMultiStatusResponse
LOCAL llOk, lnOldSel
LOCAL lnCountResult, lcHref
LOCAL loNode, loNodes, loChildNode, loChildNodes
	llOk			= .T.
	lnOldSel		= STR(SELECT(0))
	lnCountResult	= 0
	lcHref			= ""

	IF llOk AND this.lSetupOk THEN
		&& set default lMSStatus to true - VERY IMPORTANT!!!
		this.lMSStatus  = .T.
		
		&& create cursor for XML parser results
		this.buildMultiStatusCursor()
		
		&& load XML MultiStatus into DOM Parser
		IF NOT(this.oXMLDomDoc.loadXML(this.cMultiStatus))
			llOk = .F.
			=MESSAGEBOX("Es ist ein Fehler beim Laden der MultiStatus Nachricht aufgetreten. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler")
		ENDIF
		
		&& process XML Document
		IF llOk THEN
			this.oXMLDomDoc.setProperty("SelectionNamespaces", "xmlns:D='DAV:' xmlns:ox='http://www.open-xchange.org'")
			
			loNodes = this.oXMLDomDoc.selectNodes("/D:multistatus/D:response")
			IF VARTYPE(loNodes) == "O" THEN
			
				SELECT (this.cFolderCursor)
				
				FOR EACH loNode IN loNodes
					IF loNode.hasChildNodes THEN
						APPEND BLANK
						SCATTER MEMVAR

						IF VARTYPE(loNode.selectSingleNode("D:href")) == "O" THEN
							m.Href = loNode.selectSingleNode("D:href").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:status")) == "O" THEN
							m.HTTPCode = loNode.selectSingleNode("D:propstat/D:status").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:responsedescription")) == "O" THEN
							m.HTTPDesc = loNode.selectSingleNode("D:propstat/D:responsedescription").nodeTypedValue
						ENDIF
						
*!*							IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop")) == "O" THEN
*!*								m.ObjectID = loNode.selectSingleNode("D:propstat/D:prop").nodeTypedValue
*!*							ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:created_by")) == "O" THEN
							m.createdby = loNode.selectSingleNode("D:propstat/D:prop/ox:created_by").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:modified_by")) == "O" THEN
							m.modifiedby = loNode.selectSingleNode("D:propstat/D:prop/ox:modified_by").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:owner")) == "O" THEN
							m.owner = loNode.selectSingleNode("D:propstat/D:prop/ox:owner").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:creator")) == "O" THEN
							m.creator = loNode.selectSingleNode("D:propstat/D:prop/ox:creator").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:object_id")) == "O" THEN
							m.object_id = loNode.selectSingleNode("D:propstat/D:prop/ox:object_id").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:folder_id")) == "O" THEN
							m.folder_id = loNode.selectSingleNode("D:propstat/D:prop/ox:folder_id").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:module")) == "O" THEN
							m.module = loNode.selectSingleNode("D:propstat/D:prop/ox:module").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:type")) == "O" THEN
							m.type = loNode.selectSingleNode("D:propstat/D:prop/ox:type").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:title")) == "O" THEN
							m.title = loNode.selectSingleNode("D:propstat/D:prop/ox:title").nodeTypedValue
						ENDIF
						
						IF VARTYPE(loNode.selectSingleNode("D:propstat/D:prop/ox:defaultfolder")) == "O" THEN
							IF ATC("TRUE", loNode.selectSingleNode("D:propstat/D:prop/ox:defaultfolder").nodeTypedValue) > 0 THEN
								m.deffolder = .T.
							ELSE
								m.deffolder = .F.
							ENDIF
						ELSE
							m.deffolder = .F.
						ENDIF
						
						m.transfer = .F. && is this entry selected to be transfered to/from OX server?
						
						&& Save memory variables to current cursor row
						GATHER MEMVAR
						
					ENDIF && end if loNode.hasChildNodes
				ENDFOR && end foreach loNode IN loNodes
				
			ENDIF && check if loNodes is an object
		ENDIF && end if llOk...
		
	ELSE && the setup has not been called or something was wrong
		llok = this.lSetupOk
		=MESSAGEBOX("Es gab einen Fehler im Setup der WebDAV-Folders-Komponente. Die weitere Verarbeitung wurde gestoppt!", 16, "Fehler")
	ENDIF

	SELECT &lnOldSel
RETURN(llok)
ENDFUNC



*!* This method clears the object (e.g. delete the cursor)
*!*
*!* Required Parameter(s)
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
PROCEDURE clearObject
LOCAL lnOldSel
	lnOldSel		= STR(SELECT(0))
	
	SELECT (this.cFolderCursor)
	USE
	
	SELECT &lnOldSel
ENDPROC	

ENDDEFINE && END OF CLASS DEFINITION