DEFINE CLASS OPENOFFICECALC_CONTROL AS GENERIC_CONTROL OF GENERIC_CONTROL.PRG
* -------------------------------------------------------------------------------- *
*	OPENOFFICECALC_CONTROL
*	=============================================
*	Desc (short):	CSV to OpenOffice import and OpenOffice to CSV export (configurable)
*
*	Autor.......:	Juergen Ulbts (J.U.)
*	Copyright...:	2005 - Juergen Ulbts
*	Version.....:	0.12 - 2005-06-21
*	VFP Version.:	VFP 7.0
*	Description.:	This class allows to import data into a cursor and from there to different output formats
*					Most information comes from the OpenOffice Forum (there are also examples for a lot of
*					different programming languages available and a special Visual Foxpro OOoLib!
*					Forum............: http://www.oooforum.org/forum/
*					Code Snippets....: http://www.oooforum.org/forum/viewforum.phtml?f=11
*					Visual FoxPro Lib: http://kosh.datateamsys.com/~danny/OOo/VisualFoxPro-OOo/
*					
*					There is also a complete description of the OpenOffice API online.
*					API: http://api.openoffice.org/DevelopersGuide/DevelopersGuide.html
*
*
*	History.....:	Format: DATE/DATUM - DEVELOPER/ENTWICKLER (INITIALIEN) - CHANGES/AENDERUNG (multiline/mehrzeilig)
*						2005-06-08 - J.U. -	work started
*						2005-06-21 - J.U. -	updated method list
*													
*	Methods.....:	getServiceManager()
*					getDocument()
*					getApplication()
*					
*					openApplication()
*					closeApplication()
*					openAppWrkDoc(tcImportFile)
*					convertToURL(tcFilename)
*					createPropertyValue(tcName, tuValue, tnHandle, tnState)
*					createStruct(tcTypeName)
*					selectAppWrkDocColumnRow(tnColumn, tnRow)
*					getAppWrkDocSelectedElement()
*					setAppWrkDocSelectedElement(tuValue, tlFormular)
*					setAppVisible(tlVisible)
*
*					doNothing__ErrorHandler( pnError, pcErrMessage, pnLineNo, pcProgramFileSys16, pcProgram, pcErrorParamSys2018 )
*
*	TODO........:	updating list of methods
*					updating write to cell method in GENERICCONTROL and EXCELCONTROL
*					update setAppVisible(boolean) -- currently not working!!!
*                 	                                                               
* -------------------------------------------------------------------------------- *
PROTECTED oServiceManager		&& OpenOffice Service Manager
PROTECTED oDocument				&& OpenOffice document
PROTECTED oSheet				&& OpenOffice sheet in a document
PROTECTED oCell					&& OpenOffice cell in a sheet

FUNCTION getServiceManager
RETURN(this.oServiceManager)
ENDFUNC

FUNCTION getDocument
RETURN(this.oDocument)
ENDFUNC

FUNCTION getApplication
RETURN(this.oApplication)
ENDFUNC

*!* Open Application
*!* This function opens the application to be used in the following procedures and functions
*!* OpenOffice is opened here (needs ServiceManager first)
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION openApplication
LOCAL llok
	llok = DODEFAULT()
	this.oServiceManager = CREATEOBJECT("com.sun.star.ServiceManager")
	IF VARTYPE(this.oServiceManager) == "O" THEN
		this.oApplication = this.oServiceManager.createInstance("com.sun.star.frame.Desktop")
		IF VARTYPE(this.oApplication) == "O" THEN
			llOk = .T.
		ELSE
			llOk = .F.
		ENDIF
	ELSE
		llOk = .F.
	ENDIF
RETURN(llok)
ENDFUNC


*!* Close Application
*!* This function closes and removes the application that has been used
*!* Close OpenOffice
*!*
*!* Required Parameter(s):
*!*	NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION closeApplication
LOCAL llok
	llok = DODEFAULT()
	IF VARTYPE(this.oApplication) == "O" THEN
		IF VARTYPE(this.oDocument) == "O" THEN
			this.oDocument.close(.T.)
		ENDIF
		this.oApplication.Terminate()
		this.oApplication = ""
		this.oServiceManager = ""
	ENDIF
RETURN(llok)
ENDFUNC


*!* Convert To URL
*!*	This function converts a local filename to an OO-URL.
*!*
*!*	Required parameters(s)
*!*	tcFilename	= The input file (full path information) to be converted
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION convertToURL
LPARAMETERS tcFilename
LOCAL lcURL
	lcURL = ""
	IF VARTYPE(tcFilename) == "C" THEN
		&& ensure leading slash
		IF LEFT( tcFilename, 1 ) != "/"
			tcFileName = "/" + tcFileName
		ENDIF
	
		lcURL = CHRTRAN( tcFilename, "\", "/" )	&& change backslashes to forward slashes.
		lcURL = "file://" + lcURL
	ENDIF
RETURN lcURL
ENDFUNC


*!* Open Application Work Document
*!* This function opens the working document
*!*
*!* Required Parameter(s):
*!*	tcImportFile	= The input file that has to be opened and used to import the data from it
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION openAppWrkDoc
PARAMETERS tcImportFile
LOCAL llok
LOCAL ARRAY aNoArgs[1]
	aNoArgs[1] = this.createPropertyValue("Hidden", .F.)
	llok = DODEFAULT(tcImportFile)
	IF llok THEN
		&& load document
		this.oDocument = this.oApplication.loadComponentFromUrl(this.convertToURL(tcImportFile), "_blank", 0, @ aNoArgs)
	
		&& pass really an empty array
		COMARRAY(this.oDocument, 0)
		
		&& select the first sheet
		&& this.oSheet = this.oDoc.Sheets( 0)    && TODO - TEST THIS - J.U.  - 2005-06-08
		&& or
		this.oSheet = this.oDocument.getSheets().getByIndex( 0)
	ENDIF

*!*		&& ************************************************************	
*!*		&& ALTERNATIVE WAY TO GET ALL SHEETS AND SELECT ONE OF THEM!!!!
*!*		&& ************************************************************	
*!*		&& Get the collection of sheets in the document.
*!*		&& Because this collection implements both XIndexAccess and XNameAccess
*!*		&& you get lots of convenient methods such as...
*!*		&& getCount(), getByIndex(), getByName(), hasByName(), getElementNames()
*!*		oSheets = oDoc.getSheets()
*!*	   
*!*		&& Get the first sheet on the document.
*!*		oSheet = oSheets.getByIndex( 0 ) 
RETURN(llok)
ENDFUNC


*!* Create Property Value
*!* This function creates a special OpenOffice property value
*!*
*!* Required Parameter(s):
*!*	tcName		= The input value name that's the target for the value (2nd parameter)
*!* tuValue		= The undefined value (more than one type possible?)
*!*
*!* Optional Parameter(s):     
*!* tnHandle	= optional handle
*!* tnState		= optional state
*!*
FUNCTION createPropertyValue
LPARAMETERS tcName, tuValue, tnHandle, tnState
LOCAL loPropertyValue
	loPropertyValue = this.createStruct("com.sun.star.beans.PropertyValue")
	
	loPropertyValue.Name = tcName
	loPropertyValue.Value = tuValue
	
	IF TYPE([tnHandle])="N"
		loPropertyValue.Handle = tnHandle
	ENDIF
	IF TYPE([tnState])="N"
		loPropertyValue.State = tnState
	ENDIF
	
RETURN loPropertyValue
ENDFUNC



*!* Create Struct
*!* This function creates a Struct for a special typeName
*!*
*!* Required Parameter(s):
*!*	tcTypeName	= The input type name to create the struct obj
*!*
*!* Optional Parameter(s):     
*!* NONE
*!*
FUNCTION createStruct
LPARAMETERS tcTypeName
LOCAL loStruct
	loStruct = this.oServiceManager.Bridge_GetStruct( tcTypeName )
RETURN loStruct
ENDFUNC


*!* Select Application Work Document column and row
*!* This function selects the column and row of the work document
*!*
*!* Required Parameter(s):
*!*	tnColumn	= The column that should be selected
*!* tnRow		= The row that should be selected
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION selectAppWrkDocColumnRow
PARAMETERS tnColumn, tnRow
LOCAL llok
	llok = DODEFAULT(tnColumn, tnRow) && checks parameters are from the correct type
	IF llok THEN
		&& ALTERNATIVE (maybe a speed test would show which one is better)
		&& BEWARE: internal counting starts with 0 and NOT 1!!!!!!
		&& In the following line tnColumn = tnColumn-1 and tnRow = tnRow-1 (to allow counting from 1 to x)
		this.oCell = this.oSheet.getCellByPosition((tnColumn-1), (tnRow-1))
		
&&		this.oCell = this.oSheet.getCellByName(Number2Spread(tnColumn)+TRANSFORM(tnRow)) && select row and column
	ENDIF
RETURN(llok)
ENDFUNC


*!* Get Selected Element of Application Work Document
*!* This function returns the selected element of the work document
*!*
*!* Required Parameter(s):
*!* NONE
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION getAppWrkDocSelectedElement
LOCAL lcString
	lcString = DODEFAULT()
	lcString = this.oCell.String
RETURN(lcString)
ENDFUNC


*!* Set Selected Element of Application Work Document
*!* This function saves a value to the selected element of the work document
*!*
*!* Required Parameter(s):
*!* tuValue		= Value that is saved in the previously specified location
*!*
*!* Optional Parameter(s):
*!* NONE
*!*
FUNCTION setAppWrkDocSelectedElement
PARAMETERS tuValue, tlFormular
LOCAL llok
	llok = .F.
	
	IF !(VARTYPE(tlFormular) == "L") THEN && if NOT logical set it to false
		tlFormular = .F.
	ENDIF
	
	DO CASE
		CASE VARTYPE(tuValue) == "C"
			IF tlFormular THEN && check for formular...yes it is one...
				this.oCell.setFormula(tuValue)
			ELSE && no it no formular...
				this.oCell.setString(tuValue)
			ENDIF

		CASE VARTYPE(tuValue) == "N"
			this.oCell.setValue(tuValue) && TODO
	ENDCASE
	
*!*	oRanges = oRange.queryContentCells(_
*!*	  com.sun.star.sheet.CellFlags.VALUE OR _
*!*	  com.sun.star.sheet.CellFlags.DATETIME OR _
*!*	  com.sun.star.sheet.CellFlags.STRING OR _
*!*	  com.sun.star.sheet.CellFlags.FORMULA)	
RETURN(llok)
ENDFUNC



*!* Set Application Visible
*!* This function makes the application visible or invisible to the enduser
*!*
*!* Required Parameter(s):
*!* tlVisible	= Visible TRUE (.T.) or FALSE (.F.)
*!*
*!* Optional Parameter(s):     
*!*	NONE
*!*
FUNCTION setAppVisible
PARAMETERS tlVisible
LOCAL llok
	llok = DODEFAULT(tlVisible) && checks parameter if it's from the correct type
	IF llok THEN
		this.oApplication.visible = tlVisible
	ENDIF
RETURN(llok)
ENDFUNC



*!* DUMMY ERROR HANDLER
*!*
PROCEDURE doNothing__ErrorHandler( pnError, pcErrMessage, pnLineNo, pcProgramFileSys16, pcProgram, pcErrorParamSys2018 )
ENDPROC


ENDDEFINE && END OF CLASS DEFINITION
